@@ -6,7 +6,7 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: config_loader.h 713 2009-03-14 05:40:37Z takahiko $
+ * $Id: config_loader.h 1462 2011-12-21 11:55:05Z takahiko $
  */
 
 #ifndef __CONFIG_LOADER_H__
@@ -28,25 +28,25 @@ typedef enum {
 } ConfigType;
 
 typedef struct ConfigEntry {
-    const char *config_name;    // 設定項目の名前
-    const ConfigType config_type;   // 設定項目の型
-    const char *default_value;  // 設定項目のデフォルト値
-    const int struct_offset;    // 構造体の変数へのオフセット
-    const char *description;    // 設定項目の説明文
+    const char *config_name;    // name of configuration entry
+    const ConfigType config_type;   // type of the entry
+    const char *default_value;  // default value of the entry
+    const int struct_offset;    // offset of the entry
+    const char *description;    // description of the entry
 } ConfigEntry;
 
-#define CONFIG_LINE_MAX_LEN 512 // configファイルの一行の最大長
+#define CONFIG_LINE_MAX_LEN 512 // maximum line length of configuration file
 
 extern bool ConfigLoader_setConfigValue(const ConfigEntry *config_entry, const char *filename,
                                         void *config_struct);
 extern bool ConfigLoader_setDefaultValue(const ConfigEntry *config_entry, void *config_struct);
-extern bool ConfigLoader_setEqualStringOptionValue(const ConfigEntry *config_entry,
-                                                   const char *optarg, void *config_struct);
 extern bool ConfigLoader_setOptionValue(const ConfigEntry *config_entry, const char *config_key,
                                         const char *config_value, void *config_struct);
+extern bool ConfigLoader_setEqualStringOptionValue(const ConfigEntry *config_entry,
+                                                   const char *optarg, void *config_struct);
 extern void ConfigLoader_init(const ConfigEntry *config_entry, void *config_struct);
 extern void ConfigLoader_free(const ConfigEntry *config_entry, void *config_struct);
 extern void ConfigLoader_dump(const ConfigEntry *config_entry, const void *config_struct,
                               FILE *out);
 
-#endif
+#endif /* __CONFIG_LOADER_H__ */
