@@ -6,7 +6,7 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma.h 655 2009-02-28 20:47:41Z takahiko $
+ * $Id: enma.h 1098 2009-08-03 02:20:19Z takahiko $
  */
 
 #ifndef __ENMA_H__
@@ -20,6 +20,6 @@
 
 extern EnmaConfig *g_enma_config;
 extern SidfPolicy *g_sidf_policy;
-extern DkimVerifyPolicy *g_dkim_vpolicy;
+extern DkimVerificationPolicy *g_dkim_vpolicy;
 
 #endif
