@@ -6,7 +6,7 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma_dkim.h 826 2009-03-26 12:55:05Z takahiko $
+ * $Id: enma_dkim.h 1098 2009-08-03 02:20:19Z takahiko $
  */
 
 #ifndef __ENMA_DKIM_H__
@@ -17,7 +17,7 @@
 #include "dkim.h"
 #include "authresult.h"
 
-extern bool EnmaDkim_evaluate(DkimVerifySession *dkimverifier, AuthResult *authresult);
-extern bool EnmaDkimAdsp_evaluate(DkimVerifySession *dkimverifier, AuthResult *authresult);
+extern bool EnmaDkim_evaluate(DkimVerifier *dkimverifier, AuthResult *authresult);
+extern bool EnmaDkimAdsp_evaluate(DkimVerifier *dkimverifier, AuthResult *authresult);
 
 #endif /* __ENMA_DKIM_H__ */
