@@ -6,7 +6,7 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma_mfi_ctx.h 655 2009-02-28 20:47:41Z takahiko $
+ * $Id: enma_mfi_ctx.h 1098 2009-08-03 02:20:19Z takahiko $
  */
 
 #ifndef __ENMA_MFI_CTX_H__
@@ -36,11 +36,11 @@ typedef struct EnmaMfiCtx {
     char *qid;
     InetMailbox *envfrom;
     MailHeaders *headers;
-    DkimVerifySession *dkimverifier;
+    DkimVerifier *dkimverifier;
     AuthResult *authresult;
-    // Authentication-Results ヘッダを削るためのメンバ
-    int authhdr_count;          // 遭遇した Authentication-Results ヘッダの数
-    IntArray *delauthhdr;       // 何個目の Authentication-Results ヘッダを削るか
+    // for Authentication-Results headers
+    int authhdr_count;          // the number of existing Authentication-Results headers
+    IntArray *delauthhdr;       // array of Authentication-Results headers to delete
 } EnmaMfiCtx;
 
 extern EnmaMfiCtx *EnmaMfiCtx_new(void);
