@@ -6,7 +6,7 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: string_util.h 828 2009-03-27 03:20:12Z tsuruda $
+ * $Id: string_util.h 1462 2011-12-21 11:55:05Z takahiko $
  */
 
 #ifndef __STRING_UTIL_H__
@@ -19,4 +19,4 @@ extern char *strlstrip(char *string);
 extern char *strrstrip(char *string);
 extern char *strstrip(char *string);
 
-#endif
+#endif /* __STRING_UTIL_H__ */
