@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2010 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: config_loader.c 830 2009-03-27 04:44:28Z tsuruda $
+ * $Id: config_loader.c 1366 2011-10-16 08:13:40Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: config_loader.c 830 2009-03-27 04:44:28Z tsuruda $");
+RCSID("$Id: config_loader.c 1366 2011-10-16 08:13:40Z takahiko $");
 
 #include <stdio.h>
 #include <assert.h>
@@ -143,7 +143,7 @@ ConfigLoader_setLong(void *config_storage, const char *config_value)
 }
 
 /**
- * 渡された文字列情報を、syslogのfacilyを示すint型に変換し設定の保存領域に記憶
+ * 渡された文字列情報を、syslogのfacilityを示すint型に変換し設定の保存領域に記憶
  *
  * @param config_storage
  * @param config_value
@@ -589,7 +589,7 @@ ConfigLoader_setEqualStringOptionValue(const ConfigEntry *config_entry,
 }
 
 /**
- * 設定情報の記憶域の確保
+ * initialize the ConfigLoader
  *
  * @param config_entry
  * @param config_struct
@@ -633,7 +633,7 @@ ConfigLoader_init(const ConfigEntry *config_entry, void *config_struct)
 }
 
 /**
- * 設定情報の記憶領域を解放
+ * free the ConfigEntry
  *
  * @param config_entry
  * @param config_struct
@@ -681,7 +681,7 @@ ConfigLoader_free(const ConfigEntry *config_entry, void *config_struct)
 }
 
 /**
- * 現在の設定情報を表示する
+ * dump configuration entries
  *
  * @param config_entry
  * @param config_struct
