@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2011 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma.c 776 2009-03-24 03:47:22Z takahiko $
+ * $Id: enma.c 1371 2011-11-07 03:18:02Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: enma.c 776 2009-03-24 03:47:22Z takahiko $");
+RCSID("$Id: enma.c 1371 2011-11-07 03:18:02Z takahiko $");
 
 #include <stdio.h>
 #include <assert.h>
@@ -35,14 +35,14 @@ RCSID("$Id: enma.c 776 2009-03-24 03:47:22Z takahiko $");
 #include "enma.h"
 
 
-// グローバル変数を定義
-SidfPolicy *g_sidf_policy = NULL;   // sidfのポリシーオブジェクトの記憶
-DkimVerifyPolicy *g_dkim_vpolicy = NULL;
-EnmaConfig *g_enma_config = NULL;   // enmaの設定情報を記憶
+/* definition of global variable */
+SidfPolicy *g_sidf_policy = NULL;   // sidf policy variable
+DkimVerificationPolicy *g_dkim_vpolicy = NULL;  // dkim verify policy variable
+EnmaConfig *g_enma_config = NULL;   // configuration variable of ENMA
 
 
 /**
- * enmaの起動方法の説明を表示する
+ * usage of ENMA
  *
  * @param out
  */
@@ -56,7 +56,7 @@ enma_usage(FILE *out)
 
 
 /**
- * 設定情報の初期化
+ * initialize configuration
  *
  * @param argc
  * @param argv
@@ -81,8 +81,9 @@ config_init(int argc, char **argv)
 
 
 /**
- * sidfで利用する情報の初期化
+ * initialize SIDF policy
  *
+ * @param enma_config
  * @return
  */
 static SidfPolicy *
@@ -104,32 +105,40 @@ sidf_init(const EnmaConfig *enma_config)
     return sidf_policy;
 }
 
-static DkimVerifyPolicy *
+/**
+ * initialize DKIM policy
+ *
+ * @param *enma_config
+ * @return
+ */
+static DkimVerificationPolicy *
 dkim_init(const EnmaConfig *enma_config)
 {
-    dkim_stat_t set_stat;
+    DkimStatus set_stat;
 
-    DkimVerifyPolicy *dkim_vpolicy = DkimVerifyPolicy_new();
+    DkimVerificationPolicy *dkim_vpolicy = DkimVerificationPolicy_new();
     if (NULL == dkim_vpolicy) {
         return NULL;
     }   // end if
 
-    set_stat = DkimVerifyPolicy_setAuthorPriority(dkim_vpolicy, "From", ":");
+    set_stat = DkimVerificationPolicy_setAuthorPriority(dkim_vpolicy, "From", ":");
     if (DSTAT_OK != set_stat) {
         return NULL;
     }   // end if
 
-    DkimVerifyPolicy_setSignHeaderLimit(dkim_vpolicy, enma_config->dkim_signheader_limit);
-    DkimVerifyPolicy_acceptExpiredSignature(dkim_vpolicy,
-                                            enma_config->dkim_accept_expired_signature);
-    DkimVerifyPolicy_setSendmail813(dkim_vpolicy, enma_config->milter_sendmail813);
-    DkimVerifyPolicy_setLogger(dkim_vpolicy, LogHandler_syslogWithPrefix);
+    DkimVerificationPolicy_setSignHeaderLimit(dkim_vpolicy, enma_config->dkim_signheader_limit);
+    DkimVerificationPolicy_acceptExpiredSignature(dkim_vpolicy,
+                                                  enma_config->dkim_accept_expired_signature);
+    DkimVerificationPolicy_getRfc4871Compatible(dkim_vpolicy, enma_config->dkim_rfc4871_compatible);
+    DkimVerificationPolicy_supposeLeadingHeaderValueSpace(dkim_vpolicy,
+                                                          enma_config->milter_sendmail813);
+    DkimVerificationPolicy_setLogger(dkim_vpolicy, LogHandler_syslogWithPrefix);
     return dkim_vpolicy;
 }
 
 
 /**
- * メイン
+ * main function
  *
  * @param argc
  * @param argv
@@ -139,12 +148,12 @@ int
 main(int argc, char **argv)
 {
     int result = 0;
-    // 設定情報の読み込み
+    // initialize configuration
     if (0 != (result = config_init(argc, argv))) {
         ConsoleError("enma starting up failed: error=config_init failed");
         exit(result);
     }
-    // ログ出力のための初期化
+    // initialize log handler
     openlog(g_enma_config->syslog_ident, LOG_PID | LOG_NDELAY, g_enma_config->syslog_facility);
     setlogmask(LOG_UPTO(g_enma_config->syslog_logmask));
     LogHandler_init();
@@ -188,7 +197,7 @@ main(int argc, char **argv)
     }
 
     SidfPolicy_free(g_sidf_policy);
-    DkimVerifyPolicy_free(g_dkim_vpolicy);
+    DkimVerificationPolicy_free(g_dkim_vpolicy);
     EnmaConfig_free(g_enma_config);
 
     // OpenSSL cleanup
