@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2011 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma_config.c 875 2009-04-01 07:33:04Z tsuruda $
+ * $Id: enma_config.c 1371 2011-11-07 03:18:02Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: enma_config.c 875 2009-04-01 07:33:04Z tsuruda $");
+RCSID("$Id: enma_config.c 1371 2011-11-07 03:18:02Z takahiko $");
 
 #include <stdio.h>
 #include <assert.h>
@@ -32,7 +32,7 @@ RCSID("$Id: enma_config.c 875 2009-04-01 07:33:04Z tsuruda $");
 
 static ConfigEntry ConfigEntry_table[] = {
     {"milter.verbose", CONFIGTYPE_BOOLEAN, "false", offsetof(EnmaConfig, milter_verbose),
-        "verbose mode. same option as '-v' (true or false)"},
+        "verbose mode. same option as '-v' (boolean)"},
     {"milter.conffile", CONFIGTYPE_STRING, NULL, offsetof(EnmaConfig, milter_conffile),
         "configuration file on startup. same option as '-c filename' (filename)"},
     // libmilter
@@ -49,9 +49,9 @@ static ConfigEntry ConfigEntry_table[] = {
     {"milter.loglevel", CONFIGTYPE_INTEGER, "0", offsetof(EnmaConfig, milter_loglevel),
         "log level of libmilter (integer)"},
     {"milter.sendmail813", CONFIGTYPE_BOOLEAN, "false", offsetof(EnmaConfig, milter_sendmail813),
-        "compatible mode with sendmail 8.13 or earlier (true or false)"},
+        "compatible mode with sendmail 8.13 or earlier (boolean)"},
     {"milter.postfix", CONFIGTYPE_BOOLEAN, "false", offsetof(EnmaConfig, milter_postfix),
-        "use postfix's milter (true or false)"},
+        "use postfix's milter (boolean)"},
     // syslog
     {"syslog.ident", CONFIGTYPE_STRING, ENMA_MILTER_NAME, offsetof(EnmaConfig, syslog_ident),
         "syslog identifier"},
@@ -64,24 +64,26 @@ static ConfigEntry ConfigEntry_table[] = {
         "ignore source address list"},
     // spf
     {"spf.auth", CONFIGTYPE_BOOLEAN, "true", offsetof(EnmaConfig, spf_auth),
-        "enable SPF authentication (true or false)"},
+        "enable SPF authentication (boolean)"},
     {"spf.explog", CONFIGTYPE_BOOLEAN, "true", offsetof(EnmaConfig, spf_explog),
-        "record explanation of SPF (true or false)"},
+        "record explanation of SPF (boolean)"},
     // sidf
     {"sidf.auth", CONFIGTYPE_BOOLEAN, "true", offsetof(EnmaConfig, sidf_auth),
-        "enable SIDF authentication (true or false)"},
+        "enable SIDF authentication (boolean)"},
     {"sidf.explog", CONFIGTYPE_BOOLEAN, "true", offsetof(EnmaConfig, sidf_explog),
-        "record the explanation of SIDF (true or false)"},
+        "record the explanation of SIDF (boolean)"},
     // dkim
     {"dkim.auth", CONFIGTYPE_BOOLEAN, "true", offsetof(EnmaConfig, dkim_auth),
-        "enable DKIM authentication (true or false)"},
+        "enable DKIM authentication (boolean)"},
     {"dkim.signheader_limit", CONFIGTYPE_INTEGER, "10", offsetof(EnmaConfig, dkim_signheader_limit),
         "maximum number of DKIM signature headers to be verified (rests are ignored)"},
     {"dkim.accept_expired_signature", CONFIGTYPE_BOOLEAN, "false", offsetof(EnmaConfig, dkim_accept_expired_signature),
-        "accept expired dkim signature (true or false)"},
+        "accept expired dkim signature (boolean)"},
+    {"dkim.rfc4871_compatible", CONFIGTYPE_BOOLEAN, "false", offsetof(EnmaConfig, dkim_rfc4871_compatible),
+        "RFC4871 compatible mode (boolean)"},
     // dkim adsp
     {"dkimadsp.auth", CONFIGTYPE_BOOLEAN, "true", offsetof(EnmaConfig, dkimadsp_auth),
-        "enable DKIM ADSP authentication (true or false)"},
+        "enable DKIM ADSP authentication (boolean)"},
     // authresult
     {"authresult.identifier", CONFIGTYPE_STRING, "localhost", offsetof(EnmaConfig, authresult_identifier),
         "identifier of Authentication-Results header"},
@@ -92,9 +94,9 @@ static ConfigEntry ConfigEntry_table[] = {
 
 
 /**
- * configの説明を表示する
+ * usage of ENMA
  *
- * @param out	出力先のストリーム
+ * @param out: output to the stream
  */
 static void
 EnmaConfig_usage(FILE *out)
@@ -115,7 +117,7 @@ EnmaConfig_usage(FILE *out)
 
 
 /**
- * 引数で渡された設定情報を設定の保存領域に記憶
+ * set configuration entries from command-line arguments
  *
  * @param self
  * @param argc
@@ -161,7 +163,7 @@ EnmaConfig_getopt(EnmaConfig *self, int argc, char **argv)
 
 
 /**
- * 設定情報を記憶する
+ * set configuration entries
  *
  * @param self
  * @param argc
@@ -175,26 +177,26 @@ EnmaConfig_setConfig(EnmaConfig *self, int argc, char **argv)
     assert(0 < argc);
     assert(NULL != argv);
 
-    // 引数を元に設定情報を記憶
+    // set configuration entries from command-line arguments
     if (!EnmaConfig_getopt(self, argc, argv)) {
         ConsoleNotice("config getopt value set failed");
         return false;
     }
-    // 余計な引数がないかチェック
+    // check arguments error
     argc -= optind;
     argv += optind;
     if (0 < argc) {
         ConsoleNotice("too many arguments.");
         return false;
     }
-    // 設定ファイルを元に設定情報を記憶
+    // set configuration entries from configuration file
     if (NULL != g_enma_config->milter_conffile) {
         if (!ConfigLoader_setConfigValue(ConfigEntry_table, g_enma_config->milter_conffile, self)) {
             ConsoleNotice("config file load faild");
             return false;
         }
     }
-    // デフォルト値を記憶
+    // set configuration entries from default values
     if (!ConfigLoader_setDefaultValue(ConfigEntry_table, self)) {
         ConsoleNotice("config default value set failed");
         return false;
@@ -208,9 +210,9 @@ EnmaConfig_setConfig(EnmaConfig *self, int argc, char **argv)
 
 
 /**
- * EnmaConfig オブジェクトの構築
+ * initialize the EnmaConfig
  *
- * @return 各設定情報を記憶した EnmaConfig オブジェクト
+ * @return
  */
 EnmaConfig *
 EnmaConfig_new(void)
@@ -220,7 +222,7 @@ EnmaConfig_new(void)
         ConsoleError("memory allocation failed: error=%s", strerror(errno));
         return NULL;
     }
-    // 初期化するのみ
+
     ConfigLoader_init(ConfigEntry_table, self);
 
     return self;
@@ -228,16 +230,15 @@ EnmaConfig_new(void)
 
 
 /**
- * EnmaConfig オブジェクトの解放
+ * free the EnmaConfig
  *
- * @param self 解放する EnmaConfig オブジェクト
+ * @param self
  */
 void
 EnmaConfig_free(EnmaConfig *self)
 {
     assert(NULL != self);
 
-    // 解放するのみ
     ConfigLoader_free(ConfigEntry_table, self);
     free(self);
 }
