@@ -6,11 +6,11 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma_mfi.c 875 2009-04-01 07:33:04Z tsuruda $
+ * $Id: enma_mfi.c 1193 2009-09-07 00:32:32Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: enma_mfi.c 875 2009-04-01 07:33:04Z tsuruda $");
+RCSID("$Id: enma_mfi.c 1193 2009-09-07 00:32:32Z takahiko $");
 
 #include <assert.h>
 #include <errno.h>
@@ -508,28 +508,19 @@ mfi_eoh(SMFICTX *ctx)
     }
 
     if (g_enma_config->dkim_auth) {
-        // DkimVerifySession オブジェクトの初期化
-        enma_mfi_ctx->dkimverifier = DkimVerifySession_new(g_dkim_vpolicy, enma_mfi_ctx->resolver);
+        // DkimVerifier オブジェクトの初期化
+        enma_mfi_ctx->dkimverifier = DkimVerifier_new(g_dkim_vpolicy, enma_mfi_ctx->resolver);
         if (NULL == enma_mfi_ctx->dkimverifier) {
-            LogError("DkimVerifySession_new failed: err=%s", strerror(errno));
+            LogError("DkimVerifier_new failed: err=%s", strerror(errno));
             return EnmaMfi_tempfail(enma_mfi_ctx);
         }   // end if
-        dkim_stat_t set_stat =
-            DkimVerifySession_setHeaders(enma_mfi_ctx->dkimverifier, enma_mfi_ctx->headers);
-        if (DSTAT_PERMFAIL_AUTHOR_AMBIGUOUS == set_stat) {
-            LogEvent("DKIM-skip", "Author ambiguous and verification is skipped.");
-        } else if (DSTAT_ISCRITERR(set_stat)) {
-            // エラーが発生した場合
-            LogError("DkimVerifySession_setHeaders failed: err=%s", DKIM_strerror(set_stat));
-            return EnmaMfi_tempfail(enma_mfi_ctx);
-        }   // end if
-        dkim_stat_t eoh_stat = DkimVerifySession_startBody(enma_mfi_ctx->dkimverifier);
-        if (DSTAT_VERIFIED_NO_SIGNHEADER == eoh_stat) {
+        DkimStatus eoh_stat = DkimVerifier_setup(enma_mfi_ctx->dkimverifier, enma_mfi_ctx->headers);
+        if (DSTAT_INFO_NO_SIGNHEADER == eoh_stat) {
             // DKIM 署名ヘッダが1つもついていなかった場合
-            LogEvent("DKIM-skip", "No DKIM-Signature header found and verification is skipped.");
+            LogDebug("[DKIM-skip] No DKIM-Signature header found and verification is skipped.");
         } else if (DSTAT_ISCRITERR(eoh_stat)) {
             // エラーが発生した場合
-            LogError("DkimVerifySession_startBody failed: err=%s", DKIM_strerror(eoh_stat));
+            LogError("DkimVerifier_startBody failed: err=%s", DKIM_strerror(eoh_stat));
             return EnmaMfi_tempfail(enma_mfi_ctx);
         }   // end if
     }
@@ -550,11 +541,10 @@ mfi_body(SMFICTX *ctx, unsigned char *bodyp, size_t bodylen)
     }
 
     if (g_enma_config->dkim_auth) {
-        dkim_stat_t body_stat =
-            DkimVerifySession_updateBody(enma_mfi_ctx->dkimverifier, bodyp, bodylen);
+        DkimStatus body_stat = DkimVerifier_updateBody(enma_mfi_ctx->dkimverifier, bodyp, bodylen);
         if (DSTAT_ISCRITERR(body_stat)) {
             // エラーが発生した場合
-            LogError("DkimVerifySession_body failed: err=%s", DKIM_strerror(body_stat));
+            LogError("DkimVerifier_updateBody failed: err=%s", DKIM_strerror(body_stat));
             return EnmaMfi_tempfail(enma_mfi_ctx);
         }   // end if
     }   // end if
@@ -620,7 +610,7 @@ mfi_eom(SMFICTX *ctx)
         return EnmaMfi_tempfail(enma_mfi_ctx);
     }
     // Authentication-Results ヘッダをメッセージの先頭に挿入
-    if (EOK != AuthResult_status(enma_mfi_ctx->authresult)) {
+    if (0 != AuthResult_status(enma_mfi_ctx->authresult)) {
         LogError("AuthResult_status failed");
         return EnmaMfi_tempfail(enma_mfi_ctx);
     }
