@@ -6,11 +6,11 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma_mfi_ctx.c 633 2009-02-28 18:39:30Z takahiko $
+ * $Id: enma_mfi_ctx.c 1098 2009-08-03 02:20:19Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: enma_mfi_ctx.c 633 2009-02-28 18:39:30Z takahiko $");
+RCSID("$Id: enma_mfi_ctx.c 1098 2009-08-03 02:20:19Z takahiko $");
 
 #include <assert.h>
 #include <stdlib.h>
@@ -93,7 +93,7 @@ EnmaMfiCtx_reset(EnmaMfiCtx *self)
         MailHeaders_reset(self->headers);
     }
     if (NULL != self->dkimverifier) {
-        DkimVerifySession_free(self->dkimverifier);
+        DkimVerifier_free(self->dkimverifier);
         self->dkimverifier = NULL;
     }
     if (NULL != self->authresult) {
@@ -134,7 +134,7 @@ EnmaMfiCtx_free(EnmaMfiCtx *self)
         MailHeaders_free(self->headers);
     }
     if (NULL != self->dkimverifier) {
-        DkimVerifySession_free(self->dkimverifier);
+        DkimVerifier_free(self->dkimverifier);
     }
     if (NULL != self->authresult) {
         AuthResult_free(self->authresult);
