@@ -6,11 +6,11 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: enma_sidf.c 856 2009-03-30 01:09:01Z takahiko $
+ * $Id: enma_sidf.c 1460 2011-12-21 11:47:15Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: enma_sidf.c 856 2009-03-30 01:09:01Z takahiko $");
+RCSID("$Id: enma_sidf.c 1460 2011-12-21 11:47:15Z takahiko $");
 
 #include <assert.h>
 #include <stdbool.h>
@@ -29,8 +29,8 @@ RCSID("$Id: enma_sidf.c 856 2009-03-30 01:09:01Z takahiko $");
 /**
  * 必要なパラメーターが揃わず SPF 評価をスキップした場合は "permerror"
  *
- * [draft-kucherawy-sender-auth-header-20] 2.4.3.
- * permerror:  The message could not be verified due to some error which
+ * [RFC5451] 2.4.2.
+ * permerror:  The message could not be verified due to some error that
  *    is unrecoverable, such as a required header field being absent or
  *    a syntax error in a retrieved DNS TXT record.  A later attempt is
  *    unlikely to produce a final result.
@@ -293,7 +293,7 @@ EnmaSpf_evaluate(SidfPolicy *policy, DnsResolver *resolver, AuthResult *authresu
     // Sender がセットされていれば HELO で SPF/SIDF の評価がおこなわれることはない.
     if (NULL == helohost) {
         LogEvent("SPF-skip", "HELO not set, SPF-authentication skipped: ipaddr=%s", ipaddr);
-        EnmaSidfBase_appendPermError(authresult, "SPF");
+        EnmaSidfBase_appendPermError(authresult, AUTHRES_METHOD_SPF);
         return true;
     }
 
@@ -349,7 +349,7 @@ EnmaSidf_evaluate(SidfPolicy *policy, DnsResolver *resolver, AuthResult *authres
     // Sender がセットされていれば HELO で SPF/SIDF の評価がおこなわれることはない.
     if (NULL == helohost) {
         LogEvent("SIDF-skip", "HELO not set, SIDF-authentication skipped: ipaddr=%s", ipaddr);
-        EnmaSidfBase_appendPermError(authresult, "SIDF");
+        EnmaSidfBase_appendPermError(authresult, AUTHRES_METHOD_SENDERID);
         return true;
     }
     // lookup PRA header
@@ -359,7 +359,7 @@ EnmaSidf_evaluate(SidfPolicy *policy, DnsResolver *resolver, AuthResult *authres
         return false;
     }
     if (NULL == pra_mailbox) {
-        EnmaSidfBase_appendPermError(authresult, "SIDF");
+        EnmaSidfBase_appendPermError(authresult, AUTHRES_METHOD_SENDERID);
         return true;
     }
 
