@@ -6,11 +6,11 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: string_util.c 828 2009-03-27 03:20:12Z tsuruda $
+ * $Id: string_util.c 1465 2011-12-21 11:58:27Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: string_util.c 828 2009-03-27 03:20:12Z tsuruda $");
+RCSID("$Id: string_util.c 1465 2011-12-21 11:58:27Z takahiko $");
 
 #include <assert.h>
 #include <errno.h>
@@ -24,7 +24,7 @@ RCSID("$Id: string_util.c 828 2009-03-27 03:20:12Z tsuruda $");
 
 /**
  * Convert a string to a long integer (about the same strtol())
- *   different point:
+ *   differences:
  *     - does not allowed white space
  *     - only use decimal number
  *
@@ -36,7 +36,7 @@ long int
 strtolstrict(const char *string, bool *errflag)
 {
     if (0 == strlen(string) || isspace(*string)) {
-        // string is empty or includ space
+        // string is empty or included leading spaces
         *errflag = true;
         return 0;
     }
@@ -48,7 +48,7 @@ strtolstrict(const char *string, bool *errflag)
         *errflag = true;
         return 0;
     }
-    if (EINVAL == errno || ERANGE == errno || (errno != 0 && parsed_long == 0)) {
+    if (EINVAL == errno || ERANGE == errno || (0 != errno && 0 == parsed_long)) {
         *errflag = true;
         return 0;
     }
@@ -58,10 +58,10 @@ strtolstrict(const char *string, bool *errflag)
 }
 
 /**
- * 前方の空白を取り除く
+ * remove leading spaces
  *
- * @param string	処理対象の文字列
- * @return	前方の空白を取り除いた文字列へのポインタ
+ * @param string
+ * @return
  */
 char *
 strlstrip(char *string)
@@ -69,7 +69,7 @@ strlstrip(char *string)
     assert(NULL != string);
 
     char *start = string;
-    // 空白をスキップ
+    // skip leading spaces
     for (; '\0' != *start && isspace((int) (*start)); ++start);
     memmove(string, start, strlen(start) + 1);
 
@@ -78,10 +78,10 @@ strlstrip(char *string)
 
 
 /**
- * 後方の空白を取り除く
+ * remove trailing spaces
  *
- * @param string	処理対象の文字列
- * @return	後方の空白を取り除いた文字列へのポインタ
+ * @param string
+ * @return
  */
 char *
 strrstrip(char *string)
@@ -98,10 +98,10 @@ strrstrip(char *string)
 
 
 /**
- * 前後の空白を取り除く
+ * remove leading and trailing spaces
  *
- * @param string	処理対象の文字列
- * @return	前後の空白を取り除いた文字列へのポインタ
+ * @param string
+ * @return
  */
 char *
 strstrip(char *string)
