@@ -6,7 +6,7 @@
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: bindresolver.c 774 2009-03-24 02:59:35Z takahiko $
+ * $Id: bindresolver.c 1429 2011-12-03 18:35:04Z takahiko $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -14,7 +14,7 @@
 #endif
 
 #include "rcsid.h"
-RCSID("$Id: bindresolver.c 774 2009-03-24 02:59:35Z takahiko $");
+RCSID("$Id: bindresolver.c 1429 2011-12-03 18:35:04Z takahiko $");
 
 #include <stdlib.h>
 #include <string.h>
@@ -25,29 +25,22 @@ RCSID("$Id: bindresolver.c 774 2009-03-24 02:59:35Z takahiko $");
 #include <sys/socket.h>
 #include <inttypes.h>   // as a substitute for stdint.h (Solaris 9 doesn't have)
 #include <netinet/in.h>
-#include <netinet/in_systm.h>
-#include <netinet/ip.h>
 #include <netdb.h>
 #include <resolv.h>
 
-#ifndef HAVE_STRLCPY
-# include "strlcpy.h"
-#endif
-
+#include "stdaux.h"
 #include "keywordmap.h"
 #include "dnsresolv.h"
+#include "dnsresolv_internal.h"
 
-#define DNS_IP4_REVENT_SUFFIX "in-addr.arpa."
-#define DNS_IP6_REVENT_SUFFIX "ip6.arpa."
-
-#define DNS_IP4_REVENT_MAXLEN sizeof("123.456.789.012." DNS_IP4_REVENT_SUFFIX)
-#define DNS_IP6_REVENT_MAXLEN sizeof("0.1.2.3.4.5.6.7.8.9.a.b.c.d.e.f.0.1.2.3.4.5.6.7.8.9.a.b.c.d.e.f." DNS_IP6_REVENT_SUFFIX)
+#if defined(USE_LIBRESOLV) && !defined(NS_MAXMSG)
+# define NS_MAXMSG   65535   /* maximum message size */
+#endif
 
 struct DnsResolver {
     struct __res_state resolver;
     ns_msg msghanlde;
     dns_stat_t status;
-//    int resolv_errno;
     int msglen;
     unsigned char msgbuf[NS_MAXMSG];
 };
@@ -95,7 +88,7 @@ DnsResolver_free(DnsResolver *self)
     res_nclose(&self->resolver);
 #endif
     free(self);
-}   // end function : DnsResolver_free
+}   // end function: DnsResolver_free
 
 DnsResolver *
 DnsResolver_new(void)
@@ -113,63 +106,63 @@ DnsResolver_new(void)
   cleanup:
     DnsResolver_free(self);
     return NULL;
-}   // end function : DnsResolver_new
+}   // end function: DnsResolver_new
 
 size_t
 DnsAResponse_size(const DnsAResponse *self)
 {
     return self->num;
-}   // end function : DnsAResponse_free
+}   // end function: DnsAResponse_free
 
 const struct in_addr *
 DnsAResponse_addr(const DnsAResponse *self, size_t index)
 {
     return &(self->addr[index]);
-}   // end function : DnsAResponse_addr
+}   // end function: DnsAResponse_addr
 
 void
 DnsAResponse_free(DnsAResponse *self)
 {
     assert(NULL != self);
     free(self);
-}   // end function : DnsAResponse_free
+}   // end function: DnsAResponse_free
 
 size_t
 DnsAaaaResponse_size(const DnsAaaaResponse *self)
 {
     return self->num;
-}   // end function : DnsAaaaResponse_size
+}   // end function: DnsAaaaResponse_size
 
 const struct in6_addr *
 DnsAaaaResponse_addr(const DnsAaaaResponse *self, size_t index)
 {
     return &(self->addr[index]);
-}   // end function : DnsAaaaResponse_addr
+}   // end function: DnsAaaaResponse_addr
 
 void
 DnsAaaaResponse_free(DnsAaaaResponse *self)
 {
     assert(NULL != self);
     free(self);
-}   // end function : DnsAaaaResponse_free
+}   // end function: DnsAaaaResponse_free
 
 size_t
 DnsMxResponse_size(const DnsMxResponse *self)
 {
     return self->num;
-}   // end function : DnsMxResponse_size
+}   // end function: DnsMxResponse_size
 
 uint16_t
 DnsMxResponse_preference(const DnsMxResponse *self, size_t index)
 {
     return self->exchange[index]->preference;
-}   // end function : DnsMxResponse_preference
+}   // end function: DnsMxResponse_preference
 
 const char *
 DnsMxResponse_domain(const DnsMxResponse *self, size_t index)
 {
     return self->exchange[index]->domain;
-}   // end function : DnsMxResponse_domain
+}   // end function: DnsMxResponse_domain
 
 void
 DnsMxResponse_free(DnsMxResponse *self)
@@ -179,19 +172,19 @@ DnsMxResponse_free(DnsMxResponse *self)
         free(self->exchange[n]);
     }   // end for
     free(self);
-}   // end function : DnsMxResponse_free
+}   // end function: DnsMxResponse_free
 
 size_t
 DnsTxtResponse_size(const DnsTxtResponse *self)
 {
     return self->num;
-}   // end function : DnsTxtResponse_size
+}   // end function: DnsTxtResponse_size
 
 const char *
 DnsTxtResponse_data(const DnsTxtResponse *self, size_t index)
 {
     return self->data[index];
-}   // end function : DnsTxtResponse_data
+}   // end function: DnsTxtResponse_data
 
 void
 DnsTxtResponse_free(DnsTxtResponse *self)
@@ -201,37 +194,37 @@ DnsTxtResponse_free(DnsTxtResponse *self)
         free(self->data[n]);
     }   // end for
     free(self);
-}   // end function : DnsTxtResponse_free
+}   // end function: DnsTxtResponse_free
 
 size_t
 DnsSpfResponse_size(const DnsSpfResponse *self)
 {
     return self->num;
-}   // end function : DnsSpfResponse_size
+}   // end function: DnsSpfResponse_size
 
 const char *
 DnsSpfResponse_data(const DnsSpfResponse *self, size_t index)
 {
     return self->data[index];
-}   // end function : DnsSpfResponse_data
+}   // end function: DnsSpfResponse_data
 
 void
 DnsSpfResponse_free(DnsSpfResponse *self)
 {
     DnsTxtResponse_free(self);
-}   // end function : DnsSpfResponse_free
+}   // end function: DnsSpfResponse_free
 
 size_t
 DnsPtrResponse_size(const DnsPtrResponse *self)
 {
     return self->num;
-}   // end function : DnsPtrResponse_size
+}   // end function: DnsPtrResponse_size
 
 const char *
 DnsPtrResponse_domain(const DnsPtrResponse *self, size_t index)
 {
     return self->domain[index];
-}   // end function : DnsPtrResponse_domain
+}   // end function: DnsPtrResponse_domain
 
 void
 DnsPtrResponse_free(DnsPtrResponse *self)
@@ -241,7 +234,7 @@ DnsPtrResponse_free(DnsPtrResponse *self)
         free(self->domain[n]);
     }   // end for
     free(self);
-}   // end function : DnsPtrResponse_free
+}   // end function: DnsPtrResponse_free
 
 static dns_stat_t
 DnsResolver_herrno2statcode(int herrno)
@@ -262,7 +255,7 @@ DnsResolver_herrno2statcode(int herrno)
     default:
         return DNS_STAT_RESOLVER_INTERNAL;
     }   // end switch
-}   // end function : DnsResolver_herrno2statcode
+}   // end function: DnsResolver_herrno2statcode
 
 static dns_stat_t
 DnsResolver_rcode2statcode(int rcode)
@@ -283,34 +276,34 @@ DnsResolver_rcode2statcode(int rcode)
     default:
         return DNS_STAT_RESOLVER;
     }   // end switch
-}   // end function : DnsResolver_rcode2statcode
+}   // end function: DnsResolver_rcode2statcode
 
 static dns_stat_t
 DnsResolver_setHerrno(DnsResolver *self, int herrno)
 {
     self->status = DnsResolver_herrno2statcode(herrno);
     return self->status;    // for caller's convenience
-}   // end function : DnsResolver_setHerrno
+}   // end function: DnsResolver_setHerrno
 
 static dns_stat_t
 DnsResolver_setRcode(DnsResolver *self, int rcode)
 {
     self->status = DnsResolver_rcode2statcode(rcode);
     return self->status;    // for caller's convenience
-}   // end function : DnsResolver_setRcode
+}   // end function: DnsResolver_setRcode
 
 static dns_stat_t
 DnsResolver_setError(DnsResolver *self, dns_stat_t status)
 {
     self->status = status;
     return status;  // for caller's convenience
-}   // end function : DnsResolver_setError
+}   // end function: DnsResolver_setError
 
 static void
 DnsResolver_resetErrorState(DnsResolver *self)
 {
     self->status = DNS_STAT_NOERROR;
-}   // end function : DnsResolver_resetErrorState
+}   // end function: DnsResolver_resetErrorState
 
 static const char *
 DnsResolver_statcode2string(dns_stat_t status)
@@ -341,16 +334,16 @@ DnsResolver_statcode2string(dns_stat_t status)
         {NULL, 0},  // sentinel
     };
     return KeywordMap_lookupByValue(dns_stat_tbl, status);
-}   // end function : DnsResolver_statcode2string
+}   // end function: DnsResolver_statcode2string
 
 const char *
 DnsResolver_getErrorString(const DnsResolver *self)
 {
     return DnsResolver_statcode2string(self->status);
-}   // end function : DnsResolver_getErrorString
+}   // end function: DnsResolver_getErrorString
 
 /*
- * クエリを投げる.
+ * throw a DNS query and receive a response of it
  * @return
  */
 static dns_stat_t
@@ -369,7 +362,7 @@ DnsResolver_query(DnsResolver *self, const char *domain, uint16_t rrtype)
         return DnsResolver_setRcode(self, rcode_flag);
     }   // end if
     return DNS_STAT_NOERROR;
-}   // end function : DnsResolver_query
+}   // end function: DnsResolver_query
 
 dns_stat_t
 DnsResolver_lookupA(DnsResolver *self, const char *domain, DnsAResponse **resp)
@@ -414,7 +407,7 @@ DnsResolver_lookupA(DnsResolver *self, const char *domain, DnsAResponse **resp)
   nodata:
     DnsAResponse_free(respobj);
     return DnsResolver_setError(self, DNS_STAT_NODATA);
-}   // end function : DnsResolver_lookupA
+}   // end function: DnsResolver_lookupA
 
 dns_stat_t
 DnsResolver_lookupAaaa(DnsResolver *self, const char *domain, DnsAaaaResponse **resp)
@@ -459,7 +452,7 @@ DnsResolver_lookupAaaa(DnsResolver *self, const char *domain, DnsAaaaResponse **
   nodata:
     DnsAaaaResponse_free(respobj);
     return DnsResolver_setError(self, DNS_STAT_NODATA);
-}   // end function : DnsResolver_lookupAaaa
+}   // end function: DnsResolver_lookupAaaa
 
 dns_stat_t
 DnsResolver_lookupMx(DnsResolver *self, const char *domain, DnsMxResponse **resp)
@@ -493,7 +486,8 @@ DnsResolver_lookupMx(DnsResolver *self, const char *domain, DnsMxResponse **resp
         int preference = ns_get16(rdata);
         rdata += NS_INT16SZ;
 
-        // NOTE: NS_MAXDNAME で十分なのかイマイチ確証が持てないが, bind8 の dig コマンドの実装でもこの値を使っていたのでいいのではないか.
+        // NOTE: Not sure that NS_MAXDNAME is enough size of buffer for ns_name_uncompress().
+        // "dig" supplied with bind8 uses NS_MAXDNAME for this.
         char dnamebuf[NS_MAXDNAME];
         int dnamelen =
             ns_name_uncompress(self->msgbuf, self->msgbuf + self->msglen, rdata, dnamebuf,
@@ -501,17 +495,14 @@ DnsResolver_lookupMx(DnsResolver *self, const char *domain, DnsMxResponse **resp
         if (NS_INT16SZ + dnamelen != ns_rr_rdlen(rr)) {
             goto formerr;
         }   // end if
-        size_t domainlen = strlen(dnamebuf);    // ns_name_uncompress() によって dnamebuf は NULL 終端している
+        size_t domainlen = strlen(dnamebuf);    // ns_name_uncompress() terminates dnamebuf with NULL character
         respobj->exchange[respobj->num] =
             (struct mxentry *) malloc(sizeof(struct mxentry) + sizeof(char[domainlen + 1]));
         if (NULL == respobj->exchange[respobj->num]) {
             goto noresource;
         }   // end if
         respobj->exchange[respobj->num]->preference = preference;
-        if (domainlen + 1 <=
-            strlcpy(respobj->exchange[respobj->num]->domain, dnamebuf, domainlen + 1)) {
-            abort();
-        }   // end if
+        memcpy(respobj->exchange[respobj->num]->domain, dnamebuf, domainlen + 1);
         ++(respobj->num);
     }   // end for
     if (0 == respobj->num) {
@@ -531,10 +522,10 @@ DnsResolver_lookupMx(DnsResolver *self, const char *domain, DnsMxResponse **resp
   noresource:
     DnsMxResponse_free(respobj);
     return DnsResolver_setError(self, DNS_STAT_NOMEMORY);
-}   // end function : DnsResolver_lookupMx
+}   // end function: DnsResolver_lookupMx
 
 /**
- * @return 成功した場合は DNS_STAT_NOERROR.
+ * @return DNS_STAT_NOERROR on success.
  */
 static int
 DnsResolver_lookupTxtData(DnsResolver *self, uint16_t rrtype, const char *domain,
@@ -561,7 +552,8 @@ DnsResolver_lookupTxtData(DnsResolver *self, uint16_t rrtype, const char *domain
         if (rrtype != ns_rr_type(rr)) {
             continue;
         }   // end if
-        respobj->data[respobj->num] = (char *) malloc(ns_rr_rdlen(rr)); // RDLEN を越えることはない.
+        // the size of the TXT data should be smaller than RDLEN
+        respobj->data[respobj->num] = (char *) malloc(ns_rr_rdlen(rr));
         if (NULL == respobj->data[respobj->num]) {
             goto noresource;
         }   // end if
@@ -569,7 +561,7 @@ DnsResolver_lookupTxtData(DnsResolver *self, uint16_t rrtype, const char *domain
         const unsigned char *rdata_tail = ns_rr_rdata(rr) + ns_rr_rdlen(rr);
         char *bufp = respobj->data[respobj->num];
         while (rdata < rdata_tail) {
-            // 長さフィールドが RDLEN の中に収まっているか確認する
+            // check if the length octet is less than RDLEN
             if (rdata_tail < rdata + (*rdata) + 1) {
                 free(respobj->data[respobj->num]);
                 goto formerr;
@@ -578,7 +570,7 @@ DnsResolver_lookupTxtData(DnsResolver *self, uint16_t rrtype, const char *domain
             bufp += (size_t) *rdata;
             rdata += (size_t) *rdata + 1;
         }   // end while
-        *bufp = '\0';   // 扱いやすいように NULL 終端させる
+        *bufp = '\0';   // terminate with NULL
         ++(respobj->num);
     }   // end for
     if (0 == respobj->num) {
@@ -598,22 +590,22 @@ DnsResolver_lookupTxtData(DnsResolver *self, uint16_t rrtype, const char *domain
   noresource:
     DnsTxtResponse_free(respobj);
     return DnsResolver_setError(self, DNS_STAT_NOMEMORY);
-}   // end function : DnsResolver_lookupTxtData
+}   // end function: DnsResolver_lookupTxtData
 
 dns_stat_t
 DnsResolver_lookupTxt(DnsResolver *self, const char *domain, DnsTxtResponse **resp)
 {
     return DnsResolver_lookupTxtData(self, ns_t_txt, domain, resp);
-}   // end function : DnsResolver_lookupTxt
+}   // end function: DnsResolver_lookupTxt
 
 dns_stat_t
 DnsResolver_lookupSpf(DnsResolver *self, const char *domain, DnsSpfResponse **resp)
 {
     return DnsResolver_lookupTxtData(self, 99 /* as ns_t_spf */ , domain, resp);
-}   // end function : DnsResolver_lookupSpf
+}   // end function: DnsResolver_lookupSpf
 
 /*
- * buflen のサイズは DNS_IP4_REVENT_MAXLEN 以上である必要がある.
+ * @attention the size of buflen must be DNS_IP4_REVENT_MAXLEN bytes or larger
  */
 static bool
 DnsResolver_expandReverseEntry4(const struct in_addr *addr4, char *buf, size_t buflen)
@@ -622,21 +614,21 @@ DnsResolver_expandReverseEntry4(const struct in_addr *addr4, char *buf, size_t b
     int ret =
         snprintf(buf, buflen, "%hhu.%hhu.%hhu.%hhu." DNS_IP4_REVENT_SUFFIX, rawaddr[3], rawaddr[2],
                  rawaddr[1], rawaddr[0]);
-    return (ret < (int) buflen) ? true : false;
-}   // end function : DnsResolver_expandReverseEntry4
+    return bool_cast(ret < (int) buflen);
+}   // end function: DnsResolver_expandReverseEntry4
 
 /*
- * 0 以上 15 以下の整数に対応する16進数の文字を返す.
- * p が [0,15] の範囲にあることが前提. それ以外の場合の動作は未定義.
+ * Convert an integer between 0 and 15 to a corresponding ascii character between '0' and 'f'.
+ * @attention If an integer is less than 0 or greater than 15, the results are undefined.
  */
 static char
 xtoa(unsigned char p)
 {
     return p < 0xa ? p + '0' : p + 'a' - 0xa;
-}   // end function : xtoa
+}   // end function: xtoa
 
 /*
- * buflen のサイズは DNS_IP6_REVENT_MAXLEN 以上である必要がある.
+ * @attention the size of buflen must be DNS_IP6_REVENT_MAXLEN bytes or larger
  */
 static bool
 DnsResolver_expandReverseEntry6(const struct in6_addr *addr6, char *buf, size_t buflen)
@@ -653,16 +645,15 @@ DnsResolver_expandReverseEntry6(const struct in6_addr *addr6, char *buf, size_t
         *(bufp++) = xtoa(*(rawaddr++) & 0x0f);
         *(bufp++) = '.';
     }   // end for
-    memcpy(bufp, DNS_IP6_REVENT_SUFFIX, sizeof(DNS_IP6_REVENT_SUFFIX)); // 終端のNULL文字もコピーする
+    memcpy(bufp, DNS_IP6_REVENT_SUFFIX, sizeof(DNS_IP6_REVENT_SUFFIX)); // copy suffix including NULL terminator
     return true;
-}   // end function : DnsResolver_expandReverseEntry6
+}   // end function: DnsResolver_expandReverseEntry6
 
 dns_stat_t
 DnsResolver_lookupPtr(DnsResolver *self, sa_family_t sa_family, const void *addr,
                       DnsPtrResponse **resp)
 {
-    // IPv6 の逆引きエントリ名生成に十分な長さのバッファを確保する.
-    char domain[DNS_IP6_REVENT_MAXLEN];
+    char domain[DNS_IP6_REVENT_MAXLEN]; // enough size for IPv6 reverse DNS entry
     switch (sa_family) {
     case AF_INET:
         if (!DnsResolver_expandReverseEntry4(addr, domain, sizeof(domain))) {
@@ -699,7 +690,8 @@ DnsResolver_lookupPtr(DnsResolver *self, sa_family_t sa_family, const void *addr
         if (ns_t_ptr != ns_rr_type(rr)) {
             continue;
         }   // end if
-        // NOTE: NS_MAXDNAME で十分なのかイマイチ確証が持てないが, bind8 の dig コマンドの実装でもこの値を使っていたのでいいのではないか.
+        // NOTE: Not sure that NS_MAXDNAME is enough size of buffer for ns_name_uncompress().
+        // "dig" supplied with bind8 uses NS_MAXDNAME for this.
         char dnamebuf[NS_MAXDNAME];
         int dnamelen =
             ns_name_uncompress(self->msgbuf, self->msgbuf + self->msglen, ns_rr_rdata(rr), dnamebuf,
@@ -707,7 +699,7 @@ DnsResolver_lookupPtr(DnsResolver *self, sa_family_t sa_family, const void *addr
         if (dnamelen != ns_rr_rdlen(rr)) {
             goto formerr;
         }   // end if
-        respobj->domain[respobj->num] = strdup(dnamebuf);   // ns_name_uncompress() によって dnamebuf は NULL 終端している
+        respobj->domain[respobj->num] = strdup(dnamebuf);   // ns_name_uncompress() terminates dnamebuf with NULL character
         if (NULL == respobj->domain[respobj->num]) {
             goto noresource;
         }   // end if
@@ -730,4 +722,4 @@ DnsResolver_lookupPtr(DnsResolver *self, sa_family_t sa_family, const void *addr
   noresource:
     DnsPtrResponse_free(respobj);
     return DnsResolver_setError(self, DNS_STAT_NOMEMORY);
-}   // end function : DnsResolver_lookupPtr
+}   // end function: DnsResolver_lookupPtr
