@@ -1,20 +1,22 @@
 /*
- * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2008-2011 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: ldnsresolver.c 771 2009-03-24 02:08:44Z takahiko $
+ * $Id: ldnsresolver.c 1351 2011-09-21 02:22:16Z takahiko $
  */
 
+// ldns-1.6.0 or higher is required
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
 #include "rcsid.h"
-RCSID("$Id: ldnsresolver.c 771 2009-03-24 02:08:44Z takahiko $");
+RCSID("$Id: ldnsresolver.c 1351 2011-09-21 02:22:16Z takahiko $");
 
 #include <stdlib.h>
 #include <string.h>
@@ -27,23 +29,13 @@ RCSID("$Id: ldnsresolver.c 771 2009-03-24 02:08:44Z takahiko $");
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <netdb.h>
+#include <arpa/nameser.h>
 #include <ldns/ldns.h>
 
-#ifndef HAVE_STRLCPY
-# include "strlcpy.h"
-#endif
-
+#include "stdaux.h"
+#include "keywordmap.h"
 #include "dnsresolv.h"
-
-#define DNS_IP4_REVENT_SUFFIX "in-addr.arpa."
-#define DNS_IP6_REVENT_SUFFIX "ip6.arpa."
-
-#define DNS_IP4_REVENT_MAXLEN sizeof("123.456.789.012." DNS_IP4_REVENT_SUFFIX)
-#define DNS_IP6_REVENT_MAXLEN sizeof("0.1.2.3.4.5.6.7.8.9.a.b.c.d.e.f.0.1.2.3.4.5.6.7.8.9.a.b.c.d.e.f." DNS_IP6_REVENT_SUFFIX)
-
-#ifndef MAX
-#define MAX(a,b)    ((a) > (b) ? (a) : (b))
-#endif
+#include "dnsresolv_internal.h"
 
 struct DnsResolver {
     ldns_resolver *res;
@@ -51,6 +43,7 @@ struct DnsResolver {
     ldns_status res_stat;
 };
 
+// The followings are aliases of ldns_rr_list:
 struct DnsAResponse;
 struct DnsAaaaResponse;
 
@@ -80,7 +73,7 @@ DnsResolver_free(DnsResolver *self)
     assert(NULL != self);
     ldns_resolver_deep_free(self->res);
     free(self);
-}   // end function : DnsResolver_free
+}   // end function: DnsResolver_free
 
 DnsResolver *
 DnsResolver_new(void)
@@ -90,7 +83,7 @@ DnsResolver_new(void)
         return NULL;
     }   // end if
     memset(self, 0, sizeof(DnsResolver));
-    ldns_status stat = ldns_resolver_new_frm_file(&(self->res), NULL);
+    ldns_status stat = ldns_resolver_new_frm_file(&(self->res), _PATH_RESCONF);
     if (LDNS_STATUS_OK != stat) {
         goto cleanup;
     }   // end if
@@ -99,65 +92,65 @@ DnsResolver_new(void)
   cleanup:
     DnsResolver_free(self);
     return NULL;
-}   // end function : DnsResolver_new
+}   // end function: DnsResolver_new
 
 size_t
 DnsAResponse_size(const DnsAResponse *self)
 {
     return ldns_rr_list_rr_count((const ldns_rr_list *) self);
-}   // end function : DnsAResponse_free
+}   // end function: DnsAResponse_free
 
 const struct in_addr *
 DnsAResponse_addr(const DnsAResponse *self, size_t index)
 {
     return (struct in_addr *)
         ldns_rdf_data(ldns_rr_rdf(ldns_rr_list_rr((const ldns_rr_list *) self, index), 0));
-}   // end function : DnsAResponse_addr
+}   // end function: DnsAResponse_addr
 
 void
 DnsAResponse_free(DnsAResponse *self)
 {
     assert(NULL != self);
-    ldns_rr_list_free((ldns_rr_list *) self);
-}   // end function : DnsAResponse_free
+    ldns_rr_list_deep_free((ldns_rr_list *) self);
+}   // end function: DnsAResponse_free
 
 size_t
 DnsAaaaResponse_size(const DnsAaaaResponse *self)
 {
     return ldns_rr_list_rr_count((const ldns_rr_list *) self);
-}   // end function : DnsAaaaResponse_size
+}   // end function: DnsAaaaResponse_size
 
 const struct in6_addr *
 DnsAaaaResponse_addr(const DnsAaaaResponse *self, size_t index)
 {
     return (struct in6_addr *)
         ldns_rdf_data(ldns_rr_rdf(ldns_rr_list_rr((const ldns_rr_list *) self, index), 0));
-}   // end function : DnsAaaaResponse_addr
+}   // end function: DnsAaaaResponse_addr
 
 void
 DnsAaaaResponse_free(DnsAaaaResponse *self)
 {
     assert(NULL != self);
-    ldns_rr_list_free((ldns_rr_list *) self);
-}   // end function : DnsAaaaResponse_free
+    ldns_rr_list_deep_free((ldns_rr_list *) self);
+}   // end function: DnsAaaaResponse_free
 
 size_t
 DnsMxResponse_size(const DnsMxResponse *self)
 {
     return self->num;
-}   // end function : DnsMxResponse_size
+}   // end function: DnsMxResponse_size
 
 uint16_t
 DnsMxResponse_preference(const DnsMxResponse *self, size_t index)
 {
     return self->exchange[index]->preference;
-}   // end function : DnsMxResponse_preference
+}   // end function: DnsMxResponse_preference
 
 const char *
 DnsMxResponse_domain(const DnsMxResponse *self, size_t index)
 {
     return self->exchange[index]->domain;
-}   // end function : DnsMxResponse_domain
+}   // end function: DnsMxResponse_domain
 
 void
 DnsMxResponse_free(DnsMxResponse *self)
@@ -167,19 +160,19 @@ DnsMxResponse_free(DnsMxResponse *self)
         free(self->exchange[n]);
     }   // end for
     free(self);
-}   // end function : DnsMxResponse_free
+}   // end function: DnsMxResponse_free
 
 size_t
 DnsTxtResponse_size(const DnsTxtResponse *self)
 {
     return self->num;
-}   // end function : DnsTxtResponse_size
+}   // end function: DnsTxtResponse_size
 
 const char *
 DnsTxtResponse_data(const DnsTxtResponse *self, size_t index)
 {
     return self->data[index];
-}   // end function : DnsTxtResponse_data
+}   // end function: DnsTxtResponse_data
 
 void
 DnsTxtResponse_free(DnsTxtResponse *self)
@@ -189,37 +182,37 @@ DnsTxtResponse_free(DnsTxtResponse *self)
         free(self->data[n]);
     }   // end for
     free(self);
-}   // end function : DnsTxtResponse_free
+}   // end function: DnsTxtResponse_free
 
 size_t
 DnsSpfResponse_size(const DnsSpfResponse *self)
 {
     return self->num;
-}   // end function : DnsSpfResponse_size
+}   // end function: DnsSpfResponse_size
 
 const char *
 DnsSpfResponse_data(const DnsSpfResponse *self, size_t index)
 {
     return self->data[index];
-}   // end function : DnsSpfResponse_data
+}   // end function: DnsSpfResponse_data
 
 void
 DnsSpfResponse_free(DnsSpfResponse *self)
 {
     DnsTxtResponse_free(self);
-}   // end function : DnsSpfResponse_free
+}   // end function: DnsSpfResponse_free
 
 size_t
 DnsPtrResponse_size(const DnsPtrResponse *self)
 {
     return self->num;
-}   // end function : DnsPtrResponse_size
+}   // end function: DnsPtrResponse_size
 
 const char *
 DnsPtrResponse_domain(const DnsPtrResponse *self, size_t index)
 {
     return self->domain[index];
-}   // end function : DnsPtrResponse_domain
+}   // end function: DnsPtrResponse_domain
 
 void
 DnsPtrResponse_free(DnsPtrResponse *self)
@@ -229,7 +222,7 @@ DnsPtrResponse_free(DnsPtrResponse *self)
         free(self->domain[n]);
     }   // end for
     free(self);
-}   // end function : DnsPtrResponse_free
+}   // end function: DnsPtrResponse_free
 
 static dns_stat_t
 DnsResolver_rcode2statcode(ldns_pkt_rcode rcode)
@@ -261,36 +254,36 @@ DnsResolver_rcode2statcode(ldns_pkt_rcode rcode)
         return DNS_STAT_RESOLVER_INTERNAL;
     }   // end switch
 
-}   // end function : DnsResolver_rcode2statcode
+}   // end function: DnsResolver_rcode2statcode
 
 static dns_stat_t
 DnsResolver_setRcode(DnsResolver *self, ldns_pkt_rcode rcode)
 {
     self->status = DnsResolver_rcode2statcode(rcode);
-    return self->status;    // 呼び出し側の便宜のため
-}   // end function : DnsResolver_setRcode
+    return self->status;    // for caller's convenience
+}   // end function: DnsResolver_setRcode
 
 static dns_stat_t
 DnsResolver_setError(DnsResolver *self, dns_stat_t status)
 {
     self->status = status;
-    return self->status;    // 呼び出し側の便宜のため
-}   // end function : DnsResolver_setError
+    return self->status;    // for caller's convenience
+}   // end function: DnsResolver_setError
 
 static dns_stat_t
 DnsResolver_setResolverError(DnsResolver *self, ldns_status status)
 {
     self->status = DNS_STAT_RESOLVER;
     self->res_stat = status;
-    return self->status;    // 呼び出し側の便宜のため
-}   // end function : DnsResolver_setResolverError
+    return self->status;    // for caller's convenience
+}   // end function: DnsResolver_setResolverError
 
 static void
 DnsResolver_resetErrorState(DnsResolver *self)
 {
     self->status = DNS_STAT_NOERROR;
     self->res_stat = LDNS_STATUS_OK;
-}   // end function : DnsResolver_resetErrorState
+}   // end function: DnsResolver_resetErrorState
 
 static const char *
 DnsResolver_statcode2string(dns_stat_t status)
@@ -321,7 +314,7 @@ DnsResolver_statcode2string(dns_stat_t status)
         {NULL, 0},  // sentinel
     };
     return KeywordMap_lookupByValue(dns_stat_tbl, status);
-}   // end function : DnsResolver_statcode2string
+}   // end function: DnsResolver_statcode2string
 
 const char *
 DnsResolver_getErrorString(const DnsResolver *self)
@@ -329,10 +322,10 @@ DnsResolver_getErrorString(const DnsResolver *self)
     return (DNS_STAT_RESOLVER == self->status)
         ? ldns_get_errorstr_by_id(self->res_stat)
         : DnsResolver_statcode2string(self->status);
-}   // end function : DnsResolver_getErrorString
+}   // end function: DnsResolver_getErrorString
 
 /*
- * クエリを投げる.
+ * throw a DNS query and receive a response of it
  * @return
  */
 static dns_stat_t
@@ -365,19 +358,19 @@ DnsResolver_query(DnsResolver *self, const char *domain, ldns_rr_type rrtype, ld
     }   // end if
     ldns_pkt_free(packet);
     return DNS_STAT_NOERROR;
-}   // end function : DnsResolver_query
+}   // end function: DnsResolver_query
 
 dns_stat_t
 DnsResolver_lookupA(DnsResolver *self, const char *domain, DnsAResponse **resp)
 {
     return DnsResolver_query(self, domain, LDNS_RR_TYPE_A, (ldns_rr_list **) resp);
-}   // end function : DnsResolver_lookupA
+}   // end function: DnsResolver_lookupA
 
 dns_stat_t
 DnsResolver_lookupAaaa(DnsResolver *self, const char *domain, DnsAaaaResponse **resp)
 {
     return DnsResolver_query(self, domain, LDNS_RR_TYPE_AAAA, (ldns_rr_list **) resp);
-}   // end function : DnsResolver_lookupAaaa
+}   // end function: DnsResolver_lookupAaaa
 
 static bool
 DnsResolver_expandDomainName(const ldns_rdf *rdf, char *bufp, size_t buflen)
@@ -408,8 +401,9 @@ DnsResolver_expandDomainName(const ldns_rdf *rdf, char *bufp, size_t buflen)
     }   // end if
 
     uint8_t label_len = *(rdata++);
-    // rdata の長さには次のラベルの長さフィールドを考慮している.
-    // buf の長さには '.' or NULL 終端文字の分を考慮している.
+
+    // "rdata + label_len" includes the length field of the next label,
+    // and "bufp + label_len" includes '.' or NULL terminator.
     while (rdata + label_len < rdata_tail && bufp + label_len < buf_tail) {
         memcpy(bufp, rdata, label_len);
         rdata += label_len;
@@ -422,7 +416,7 @@ DnsResolver_expandDomainName(const ldns_rdf *rdf, char *bufp, size_t buflen)
         *(bufp++) = '.';
     }   // end while
     return false;
-}   // end function : DnsResolver_expandDomainName
+}   // end function: DnsResolver_expandDomainName
 
 dns_stat_t
 DnsResolver_lookupMx(DnsResolver *self, const char *domain, DnsMxResponse **resp)
@@ -436,7 +430,7 @@ DnsResolver_lookupMx(DnsResolver *self, const char *domain, DnsMxResponse **resp
     DnsMxResponse *respobj =
         (DnsMxResponse *) malloc(sizeof(DnsMxResponse) + rr_count * sizeof(struct mxentry *));
     if (NULL == respobj) {
-        ldns_rr_list_free(rrlist);
+        ldns_rr_list_deep_free(rrlist);
         return DnsResolver_setError(self, DNS_STAT_NOMEMORY);
     }   // end if
     memset(respobj, 0, sizeof(DnsPtrResponse) + rr_count * sizeof(struct mxentry *));
@@ -472,27 +466,27 @@ DnsResolver_lookupMx(DnsResolver *self, const char *domain, DnsMxResponse **resp
         goto nodata;
     }   // end if
     *resp = respobj;
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     return DNS_STAT_NOERROR;
 
   formerr:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsMxResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_FORMERR);
 
   nodata:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsMxResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_NODATA);
 
   noresource:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsMxResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_NOMEMORY);
-}   // end function : DnsResolver_lookupMx
+}   // end function: DnsResolver_lookupMx
 
 /**
- * @return 成功した場合は DNS_STAT_NOERROR.
+ * @return DNS_STAT_NOERROR on success.
  */
 static dns_stat_t
 DnsResolver_lookupTxtData(DnsResolver *self, ldns_rr_type rrtype, const char *domain,
@@ -507,7 +501,7 @@ DnsResolver_lookupTxtData(DnsResolver *self, ldns_rr_type rrtype, const char *do
     DnsTxtResponse *respobj =
         (DnsTxtResponse *) malloc(sizeof(DnsTxtResponse) + rr_count * sizeof(char *));
     if (NULL == respobj) {
-        ldns_rr_list_free(rrlist);
+        ldns_rr_list_deep_free(rrlist);
         return DnsResolver_setError(self, DNS_STAT_NOMEMORY);
     }   // end if
     memset(respobj, 0, sizeof(DnsTxtResponse) + rr_count * sizeof(char *));
@@ -549,39 +543,39 @@ DnsResolver_lookupTxtData(DnsResolver *self, ldns_rr_type rrtype, const char *do
         goto nodata;
     }   // end if
     *resp = respobj;
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     return DNS_STAT_NOERROR;
 
   formerr:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsTxtResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_FORMERR);
 
   nodata:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsTxtResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_NODATA);
 
   noresource:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsTxtResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_NOMEMORY);
-}   // end function : DnsResolver_lookupTxtData
+}   // end function: DnsResolver_lookupTxtData
 
 dns_stat_t
 DnsResolver_lookupTxt(DnsResolver *self, const char *domain, DnsTxtResponse **resp)
 {
     return DnsResolver_lookupTxtData(self, LDNS_RR_TYPE_TXT, domain, resp);
-}   // end function : DnsResolver_lookupTxt
+}   // end function: DnsResolver_lookupTxt
 
 dns_stat_t
 DnsResolver_lookupSpf(DnsResolver *self, const char *domain, DnsSpfResponse **resp)
 {
     return DnsResolver_lookupTxtData(self, LDNS_RR_TYPE_SPF, domain, resp);
-}   // end function : DnsResolver_lookupSpf
+}   // end function: DnsResolver_lookupSpf
 
 /*
- * buflen のサイズは DNS_IP4_REVENT_MAXLEN 以上である必要がある.
+ * @attention the size of buflen must be DNS_IP4_REVENT_MAXLEN bytes or larger
  */
 static bool
 DnsResolver_expandReverseEntry4(const struct in_addr *addr4, char *buf, size_t buflen)
@@ -590,21 +584,21 @@ DnsResolver_expandReverseEntry4(const struct in_addr *addr4, char *buf, size_t b
     int ret =
         snprintf(buf, buflen, "%hhu.%hhu.%hhu.%hhu." DNS_IP4_REVENT_SUFFIX, rawaddr[3], rawaddr[2],
                  rawaddr[1], rawaddr[0]);
-    return (ret < (int) buflen) ? true : false;
-}   // end function : DnsResolver_expandReverseEntry4
+    return bool_cast(ret < (int) buflen);
+}   // end function: DnsResolver_expandReverseEntry4
 
 /*
- * 0 以上 15 以下の整数に対応する16進数の文字を返す.
- * p が [0,15] の範囲にあることが前提. それ以外の場合の動作は未定義.
+ * Convert an integer between 0 and 15 to a corresponding ascii character between '0' and 'f'.
+ * @attention If an integer is less than 0 or greater than 15, the results are undefined.
  */
 static char
 xtoa(unsigned char p)
 {
     return p < 0xa ? p + '0' : p + 'a' - 0xa;
-}   // end function : xtoa
+}   // end function: xtoa
 
 /*
- * buflen のサイズは DNS_IP6_REVENT_MAXLEN 以上である必要がある.
+ * @attention the size of buflen must be DNS_IP6_REVENT_MAXLEN bytes or larger
  */
 static bool
 DnsResolver_expandReverseEntry6(const struct in6_addr *addr6, char *buf, size_t buflen)
@@ -621,16 +615,15 @@ DnsResolver_expandReverseEntry6(const struct in6_addr *addr6, char *buf, size_t
         *(bufp++) = xtoa(*(rawaddr++) & 0x0f);
         *(bufp++) = '.';
     }   // end for
-    memcpy(bufp, DNS_IP6_REVENT_SUFFIX, sizeof(DNS_IP6_REVENT_SUFFIX)); // 終端のNULL文字もコピーする
+    memcpy(bufp, DNS_IP6_REVENT_SUFFIX, sizeof(DNS_IP6_REVENT_SUFFIX)); // copy suffix including NULL terminator
     return true;
-}   // end function : DnsResolver_expandReverseEntry6
+}   // end function: DnsResolver_expandReverseEntry6
 
 dns_stat_t
 DnsResolver_lookupPtr(DnsResolver *self, sa_family_t sa_family, const void *addr,
                       DnsPtrResponse **resp)
 {
-    // IPv6 の逆引きエントリ名生成に十分な長さのバッファを確保する.
-    char domain[DNS_IP6_REVENT_MAXLEN];
+    char domain[DNS_IP6_REVENT_MAXLEN]; // enough size for IPv6 reverse DNS entry
     switch (sa_family) {
     case AF_INET:
         if (!DnsResolver_expandReverseEntry4(addr, domain, sizeof(domain))) {
@@ -655,7 +648,7 @@ DnsResolver_lookupPtr(DnsResolver *self, sa_family_t sa_family, const void *addr
     DnsPtrResponse *respobj =
         (DnsPtrResponse *) malloc(sizeof(DnsPtrResponse) + rr_count * sizeof(char *));
     if (NULL == respobj) {
-        ldns_rr_list_free(rrlist);
+        ldns_rr_list_deep_free(rrlist);
         return DnsResolver_setError(self, DNS_STAT_NOMEMORY);
     }   // end if
     memset(respobj, 0, sizeof(DnsPtrResponse) + rr_count * sizeof(char *));
@@ -687,21 +680,21 @@ DnsResolver_lookupPtr(DnsResolver *self, sa_family_t sa_family, const void *addr
         goto nodata;
     }   // end if
     *resp = respobj;
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     return DNS_STAT_NOERROR;
 
   formerr:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsPtrResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_FORMERR);
 
   nodata:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsPtrResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_NODATA);
 
   noresource:
-    ldns_rr_list_free(rrlist);
+    ldns_rr_list_deep_free(rrlist);
     DnsPtrResponse_free(respobj);
     return DnsResolver_setResolverError(self, DNS_STAT_NOMEMORY);
-}   // end function : DnsResolver_lookupPtr
+}   // end function: DnsResolver_lookupPtr
