@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2007-2011 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: sidfmacro.c 847 2009-03-29 13:34:25Z takahiko $
+ * $Id: sidfmacro.c 1343 2011-07-30 19:21:50Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: sidfmacro.c 847 2009-03-29 13:34:25Z takahiko $");
+RCSID("$Id: sidfmacro.c 1343 2011-07-30 19:21:50Z takahiko $");
 
 #include <stdbool.h>
 #include <stdio.h>
@@ -24,6 +24,7 @@ RCSID("$Id: sidfmacro.c 847 2009-03-29 13:34:25Z takahiko $");
 #include <arpa/nameser.h>
 
 #include "sidflogger.h"
+#include "stdaux.h"
 #include "ptrop.h"
 #include "pstring.h"
 #include "xbuffer.h"
@@ -36,10 +37,6 @@ RCSID("$Id: sidfmacro.c 847 2009-03-29 13:34:25Z takahiko $");
 #include "sidfrequest.h"
 #include "sidfmacro.h"
 
-#ifndef MIN
-#define MIN(a,b) (((a)<(b))?(a):(b))
-#endif
-
 #define SIDF_MACRO_DOTTED_INET6ADDRLEN (sizeof("0.1.2.3.4.5.6.7.8.9.a.b.c.d.e.f.0.1.2.3.4.5.6.7.8.9.a.b.c.d.e.f."))
 
 #define IS_MACRO_LITERAL(c) ((0x21 <= (c) && (c) <= 0x7e) && '%' != (c))
@@ -88,8 +85,11 @@ SidfMacro_init(SidfMacro *self)
     self->reverse = false;
     self->url_escape = false;
     self->transformer = 0;
-}   // end function : SidfMacro_init
+}   // end function: SidfMacro_init
 
+/**
+ * @attention The returned string should be released with free() when no longer needed.
+ */
 static char *
 SidfMacro_dupMailboxAsString(const InetMailbox *mailbox)
 {
@@ -106,137 +106,127 @@ SidfMacro_dupMailboxAsString(const InetMailbox *mailbox)
     memcpy(mailaddr + localpart_len + 1, domainpart, domainpart_len);
     mailaddr[localpart_len + domainpart_len + 1] = '\0';
     return mailaddr;
-}   // end function : SidfMacro_dupMailboxAsString
+}   // end function: SidfMacro_dupMailboxAsString
 
 /**
- * @param request SidfRequest object.
- * @param revdomain
- * @return 1 if IP addresses match.
- *         0 if IP addresses don't match.
- *         -1 if DNS error occurred.
+ * @attention The returned string should be released with free() when no longer needed.
  */
-static bool
-SidfMacro_isValidatedDomainName4(const SidfRequest *request, const char *revdomain)
-{
-    DnsAResponse *resp;
-    dns_stat_t query_stat = DnsResolver_lookupA(request->resolver, revdomain, &resp);
-    if (DNS_STAT_NOERROR != query_stat) {
-        return -1;
-    }   // end if
-    for (size_t m = 0; m < DnsAResponse_size(resp); ++m) {
-        if (0 == memcmp(DnsAResponse_addr(resp, m), &(request->ipaddr.addr4), NS_INADDRSZ)) {
-            DnsAResponse_free(resp);
-            return 1;
-        }   // end if
-    }   // end for
-    DnsAResponse_free(resp);
-    return 0;
-}   // end function : SidfMacro_isValidatedDomainName4
-
-/**
- * @param request SidfRequest object.
- * @param revdomain
- * @return 1 if IP addresses match.
- *         0 if IP addresses don't match.
- *         -1 if DNS error occurred.
- */
-static bool
-SidfMacro_isValidatedDomainName6(const SidfRequest *request, const char *revdomain)
-{
-    DnsAaaaResponse *resp;
-    dns_stat_t query_stat = DnsResolver_lookupAaaa(request->resolver, revdomain, &resp);
-    if (DNS_STAT_NOERROR != query_stat) {
-        return -1;
-    }   // end if
-    for (size_t m = 0; m < DnsAaaaResponse_size(resp); ++m) {
-        if (0 == memcmp(DnsAaaaResponse_addr(resp, m), &(request->ipaddr.addr6), NS_IN6ADDRSZ)) {
-            DnsAaaaResponse_free(resp);
-            return 1;
-        }   // end if
-    }   // end for
-    DnsAaaaResponse_free(resp);
-    return 0;
-}   // end function : SidfMacro_isValidatedDomainName6
-
-/*
- * @return マクロの値が決定した場合 (IPアドレスが一致した場合 or DNS エラーが発生した場合) は true,
- *         IP アドレスが一致しなかった場合は false.
- */
-static bool
-SidfMacro_isValidatedDomainName(const SidfRequest *request, const char *revdomain, char **expand)
-{
-    int ret;
-    switch (request->sa_family) {
-    case AF_INET:
-        ret = SidfMacro_isValidatedDomainName4(request, revdomain);
-        break;
-    case AF_INET6:
-        ret = SidfMacro_isValidatedDomainName6(request, revdomain);
-        break;
-    default:
-        abort();
-    }   // end switch
-    switch (ret) {
-    case 1:
-        *expand = strdup(revdomain);
-        return true;
-    case 0:
-        return false;
-    case -1:
-        *expand = strdup(SIDF_MACRO_DEFAULT_P_MACRO_VALUE);
-        return true;
-    default:
-        abort();
-    }   // end switch
-}   // end function : SidfMacro_isValidatedDomainName
-
 static char *
 SidfMacro_dupValidatedDomainName(const SidfRequest *request, const char *domain)
 {
+    /*
+     * [RFC4408] 8.1.
+     * The "p" macro expands to the validated domain name of <ip>.  The
+     * procedure for finding the validated domain name is defined in Section
+     * 5.5.  If the <domain> is present in the list of validated domains, it
+     * SHOULD be used.  Otherwise, if a subdomain of the <domain> is
+     * present, it SHOULD be used.  Otherwise, any name from the list may be
+     * used.  If there are no validated domain names or if a DNS error
+     * occurs, the string "unknown" is used.
+     */
+
     DnsPtrResponse *respptr;
     dns_stat_t ptrquery_stat =
         DnsResolver_lookupPtr(request->resolver, request->sa_family, &(request->ipaddr), &respptr);
     if (DNS_STAT_NOERROR != ptrquery_stat) {
         return strdup(SIDF_MACRO_DEFAULT_P_MACRO_VALUE);
     }   // end if
+
+    // TODO: stable sort をする代わりにリストを3回なめている. stable sort をする方がエレガント.
+    size_t resp_num_limit =
+        MIN(DnsPtrResponse_size(respptr), SIDF_MACRO_DOMAIN_VALIDATION_PTRRR_MAXNUM);
     char *expand = NULL;
-    for (size_t n = 0;
-         n < MIN(DnsPtrResponse_size(respptr), SIDF_MACRO_DOMAIN_VALIDATION_PTRRR_MAXNUM); ++n) {
-        if (InetDomain_equals(domain, DnsPtrResponse_domain(respptr, n))
-            && SidfMacro_isValidatedDomainName(request, DnsPtrResponse_domain(respptr, n), &expand)) {
-            goto finally;
+
+    /*
+     * [RFC4408] 8.1.
+     * If the <domain> is present in the list of validated domains, it SHOULD be used.
+     */
+    for (size_t n = 0; n < resp_num_limit; ++n) {
+        const char *revdomain = DnsPtrResponse_domain(respptr, n);
+        if (InetDomain_equals(domain, revdomain)) {
+            switch (SidfRequest_isValidatedDomainName(request, revdomain)) {
+            case 1:
+                expand = strdup(revdomain);
+                goto finally;
+            case 0:
+                // do nothing
+                break;
+            case -1:
+                goto use_default_macro_value;
+            default:
+                abort();
+            }   // end switch
         }   // end if
     }   // end for
-    for (size_t n = 0;
-         n < MIN(DnsPtrResponse_size(respptr), SIDF_MACRO_DOMAIN_VALIDATION_PTRRR_MAXNUM); ++n) {
-        if (InetDomain_isParent(domain, DnsPtrResponse_domain(respptr, n))
-            && !InetDomain_equals(domain, DnsPtrResponse_domain(respptr, n))
-            && SidfMacro_isValidatedDomainName(request, DnsPtrResponse_domain(respptr, n), &expand)) {
-            goto finally;
+
+    /*
+     * [RFC4408] 8.1.
+     * Otherwise, if a subdomain of the <domain> is present, it SHOULD be used.
+     */
+    for (size_t n = 0; n < resp_num_limit; ++n) {
+        const char *revdomain = DnsPtrResponse_domain(respptr, n);
+        if (InetDomain_isParent(domain, revdomain) && !InetDomain_equals(domain, revdomain)) {
+            switch (SidfRequest_isValidatedDomainName(request, revdomain)) {
+            case 1:
+                expand = strdup(revdomain);
+                goto finally;
+            case 0:
+                // do nothing
+                break;
+            case -1:
+                goto use_default_macro_value;
+            default:
+                abort();
+            }   // end switch
         }   // end if
     }   // end for
-    for (size_t n = 0;
-         n < MIN(DnsPtrResponse_size(respptr), SIDF_MACRO_DOMAIN_VALIDATION_PTRRR_MAXNUM); ++n) {
-        if (!InetDomain_isParent(domain, DnsPtrResponse_domain(respptr, n))
-            && SidfMacro_isValidatedDomainName(request, DnsPtrResponse_domain(respptr, n), &expand)) {
-            goto finally;
+
+    /*
+     * [RFC4408] 8.1.
+     * Otherwise, any name from the list may be used.
+     */
+    for (size_t n = 0; n < resp_num_limit; ++n) {
+        const char *revdomain = DnsPtrResponse_domain(respptr, n);
+        if (!InetDomain_isParent(domain, revdomain)) {
+            switch (SidfRequest_isValidatedDomainName(request, revdomain)) {
+            case 1:
+                expand = strdup(revdomain);
+                goto finally;
+            case 0:
+                // do nothing
+                break;
+            case -1:
+                goto use_default_macro_value;
+            default:
+                abort();
+            }   // end switch
         }   // end if
     }   // end for
 
+  use_default_macro_value:
+    /*
+     * [RFC4408] 8.1.
+     * If there are no validated domain names or if a DNS error occurs, the string "unknown" is used.
+     */
+    expand = strdup(SIDF_MACRO_DEFAULT_P_MACRO_VALUE);
+
   finally:
     DnsPtrResponse_free(respptr);
     return expand;
-}   // end function : SidfMacro_dupValidatedDomainName
+}   // end function: SidfMacro_dupValidatedDomainName
 
 /*
- * p が [0,15] の範囲にあることが前提. それ以外の場合の動作は未定義.
+ * The argument must be in the range 0-15, otherwise the behavior is undefined.
  */
 static char
 xtoa(unsigned char p)
 {
     return p < 0xa ? p + '0' : p + 'a' - 0xa;
-}   // end function : xtoa
+}   // end function: xtoa
 
+/**
+ * @attention The returned string should be released with free() when no longer needed.
+ */
 static char *
 SidfMacro_dupDottedIpAddr(const SidfRequest *request)
 {
@@ -257,12 +247,14 @@ SidfMacro_dupDottedIpAddr(const SidfRequest *request)
             *(bufp++) = '.';
         }   // end for
         return strpdup(addrbuf6, bufp - 1);
-        break;
     default:
         abort();
     }   // end switch
-}   // end function : SidfMacro_dupDottedIpAddr
+}   // end function: SidfMacro_dupDottedIpAddr
 
+/**
+ * @attention The returned string should be released with free() when no longer needed.
+ */
 static char *
 SidfMacro_dupMacroSource(const SidfRequest *request, SidfMacroLetter macro_letter)
 {
@@ -297,7 +289,7 @@ SidfMacro_dupMacroSource(const SidfRequest *request, SidfMacroLetter macro_lette
     default:
         abort();
     }   // end switch
-}   // end function : SidfMacro_dupMacroSource
+}   // end function: SidfMacro_dupMacroSource
 
 /**
  * 与えられた文字列の領域をセパレーターで区切り, 2次元配列を構築する.
@@ -305,8 +297,8 @@ SidfMacro_dupMacroSource(const SidfRequest *request, SidfMacroLetter macro_lette
  * @param s 区切りたい文字列
  * @param delimstr デリミタとして使う文字を繋げたNULL終端文字列
  * @param num 要素の数を受け取る変数へのポインタ
- * @attention s は上書きされる
- * @attention 返値は使用後に free() を用いて解放すること
+ * @attention s will be overwritten
+ * @attention The returned string should be released with free() when no longer needed.
  */
 static char **
 SidfMacro_splitMacroSource(char *s, const char *delimstr, size_t *num)
@@ -331,7 +323,7 @@ SidfMacro_splitMacroSource(char *s, const char *delimstr, size_t *num)
     r[++idx] = NULL;
     *num = idx;
     return r;
-}   // end function : SidfMacro_splitMacroSource
+}   // end function: SidfMacro_splitMacroSource
 
 static SidfStat
 SidfMacro_expandMacro(const SidfMacro *macro, const SidfRequest *request, XBuffer *xbuf)
@@ -372,7 +364,7 @@ SidfMacro_expandMacro(const SidfMacro *macro, const SidfRequest *request, XBuffe
     free(macro_parts);
     free(macro_source);
     return SIDF_STAT_OK;
-}   // end function : SidfMacro_expandMacro
+}   // end function: SidfMacro_expandMacro
 
 /*
  * [RFC4408]
@@ -411,7 +403,7 @@ SidfMacro_parseDelimiterBlock(SidfMacro *macro, const SidfRequest *request, cons
     *delims_tail = '\0';
     *nextp = p;
     return SIDF_STAT_OK;
-}   // end function : SidfMacro_parseDelimiterBlock
+}   // end function: SidfMacro_parseDelimiterBlock
 
 /*
  * [RFC4408]
@@ -442,7 +434,7 @@ SidfMacro_parseMacroLetter(SidfMacro *macro, const SidfRequest *request, const c
                  * are then URL escaped.  URL escaping must be performed for characters
                  * not in the "uric" set, which is defined in [RFC3986].
                  */
-                macro->url_escape = isupper(*head) ? true : false;
+                macro->url_escape = bool_cast(isupper(*head));
                 *nextp = head + 1;
                 return SIDF_STAT_OK;
             }   // end if
@@ -454,7 +446,7 @@ SidfMacro_parseMacroLetter(SidfMacro *macro, const SidfRequest *request, const c
     SidfLogPermFail(request->policy, "macro-letter not specified");
     *nextp = head;
     return SIDF_STAT_RECORD_SYNTAX_VIOLATION;
-}   // end function : SidfMacro_parseMacroLetter
+}   // end function: SidfMacro_parseMacroLetter
 
 /*
  * [RFC4408]
@@ -474,10 +466,10 @@ SidfMacro_parseTransformers(SidfMacro *macro, const char *head, const char *tail
     // 数字を含まない場合 strptoul は 0 を返す.
     // transformer は 0 の場合無制限を表すので数字を含まない場合のハンドルは (たまたま) 必要ない.
     macro->transformer = strptoul(p, tail, &p);
-    macro->reverse = (0 < XSkip_char(p, tail, 'r', &p)) ? true : false;
+    macro->reverse = bool_cast(0 < XSkip_char(p, tail, 'r', &p));
     *nextp = p;
     return p - head;
-}   // end function : SidfMacro_parseTransformers
+}   // end function: SidfMacro_parseTransformers
 
 /*
  * @return SIDF_STAT_OK: 1文字以上マッチ
@@ -578,7 +570,7 @@ SidfMacro_parseMacroExpand(const SidfRequest *request, const char *head, const c
     }   // end if
     *nextp = head;
     return SIDF_STAT_RECORD_NOT_MATCH;
-}   // end function : SidfMacro_parseMacroExpand
+}   // end function: SidfMacro_parseMacroExpand
 
 /*
  * [RFC4408]
@@ -597,7 +589,7 @@ SidfMacro_parseMacroLiteralBlock(const char *head, const char *tail, const char
         XBuffer_appendStringN(xbuf, head, matchlen);
     }   // end if
     return matchlen;
-}   // end function : SidfMacro_parseMacroLiteralBlock
+}   // end function: SidfMacro_parseMacroLiteralBlock
 
 /*
  * [RFC4408]
@@ -605,24 +597,26 @@ SidfMacro_parseMacroLiteralBlock(const char *head, const char *tail, const char
  */
 static SidfStat
 SidfMacro_parseMacroString(const SidfRequest *request, const char *head, const char *tail,
-                           bool exp_record, const char **nextp, XBuffer *xbuf)
+                           bool exp_record, const char **nextp, bool *literal_terminated,
+                           XBuffer *xbuf)
 {
     const char *p = head;
     while (true) {
-        (void) SidfMacro_parseMacroLiteralBlock(p, tail, &p, xbuf);
+        int literal_len = SidfMacro_parseMacroLiteralBlock(p, tail, &p, xbuf);
         SidfStat macro_stat = SidfMacro_parseMacroExpand(request, p, tail, exp_record, &p, xbuf);
         switch (macro_stat) {
         case SIDF_STAT_OK:
             break;
         case SIDF_STAT_RECORD_NOT_MATCH:
             *nextp = p;
+            SETDEREF(literal_terminated, (0 < literal_len) ? true : false);
             return (0 < p - head) ? SIDF_STAT_OK : SIDF_STAT_RECORD_NOT_MATCH;
         default:
             *nextp = head;
             return macro_stat;
         }   // end switch
     }   // end while
-}   // end function : SidfMacro_parseMacroString
+}   // end function: SidfMacro_parseMacroString
 
 /*
  * [RFC4408]
@@ -635,7 +629,7 @@ SidfMacro_parseExplainString(const SidfRequest *request, const char *head, const
     const char *p = head;
     while (true) {
         int sp_match = XParse_char(p, tail, ' ', &p, xbuf);
-        SidfStat parse_stat = SidfMacro_parseMacroString(request, p, tail, true, &p, xbuf);
+        SidfStat parse_stat = SidfMacro_parseMacroString(request, p, tail, true, &p, NULL, xbuf);
         switch (parse_stat) {
         case SIDF_STAT_OK:
             break;
@@ -650,45 +644,47 @@ SidfMacro_parseExplainString(const SidfRequest *request, const char *head, const
             return parse_stat;
         }   // end switch
     }   // end while
-}   // end function : SidfMacro_parseExplainString
+}   // end function: SidfMacro_parseExplainString
 
 /*
  * [RFC4408]
+ * domain-end       = ( "." toplabel [ "." ] ) / macro-expand
  * toplabel         = ( *alphanum ALPHA *alphanum ) /
  *                    ( 1*alphanum "-" *( alphanum / "-" ) alphanum )
  *                    ; LDH rule plus additional TLD restrictions
  *                    ; (see [RFC3696], Section 2)
  */
 static int
-SidfMacro_skipTopLabel(const char *head, const char *tail, const char **nextp)
+SidfMacro_skipbackTopLabel(const char *head, const char *tail, const char **prevp)
 {
-    // ABNF をよく読むと　2821 sub-domain と同じ
-    return XSkip_subDomain(head, tail, nextp);
-}   // end function : SidfMacro_skipTopLabel
-
-/*
- * [RFC4408]
- * domain-end       = ( "." toplabel [ "." ] ) / macro-expand
- */
-static SidfStat
-SidfMacro_parseDomainEnd(const SidfRequest *request, const char *head, const char *tail,
-                         const char **nextp, XBuffer *xbuf)
-{
-    const char *p;
-    if (0 < XSkip_char(head, tail, '.', &p) && 0 < SidfMacro_skipTopLabel(p, tail, &p)) {
-        XSkip_char(p, tail, '.', nextp);
-        XBuffer_appendStringN(xbuf, head, *nextp - head);
-        return SIDF_STAT_OK;
+    const char *q = tail - 1;
+    *prevp = tail;
+    if (head <= q && '.' == *q) {
+        --q;
     }   // end if
-    return SidfMacro_parseMacroExpand(request, head, tail, false, nextp, xbuf);
-}   // end function : SidfMacro_parseDomainEnd
+
+    if (q < head || !IS_LET_DIG(*q)) {
+        return 0;
+    }   // end if
+
+    for (--q; head <= q; --q) {
+        if (IS_LET_DIG(*q) || '-' == *q) {
+            continue;
+        } else if ('.' == *q && '-' != *(q + 1)) {
+            *prevp = q;
+            return tail - q;
+        }   // end if
+        return 0;
+    }   // end for
+}   // end function: SidfMacro_skipbackTopLabel
 
 /*
  * [RFC4408]
  * domain-spec      = macro-string domain-end
  * domain-end       = ( "." toplabel [ "." ] ) / macro-expand
- * toplabel は RFC2821 の sub-domain に等しい
+ * (toplabel is equal to sub-domain of RFC5321)
  *
+ * we obtain the following:
  * domain-spec      = *( macro-expand / macro-literal ) ( ( "." sub-domain [ "." ] ) / macro-expand )
  */
 SidfStat
@@ -707,21 +703,27 @@ SidfMacro_parseDomainSpec(const SidfRequest *request, const char *head, const ch
 // defined in Section 5.
 {
     const char *p = head;
-
-    SidfStat parse_stat = SidfMacro_parseMacroString(request, p, tail, false, &p, xbuf);
+    bool literal_terminated;
+    SidfStat parse_stat =
+        SidfMacro_parseMacroString(request, p, tail, false, &p, &literal_terminated, xbuf);
     if (SIDF_STAT_OK != parse_stat) {
         *nextp = head;
         return parse_stat;
     }   // end if
 
-    parse_stat = SidfMacro_parseDomainEnd(request, p, tail, &p, xbuf);
-    if (SIDF_STAT_OK == parse_stat || SIDF_STAT_RECORD_NOT_MATCH == parse_stat) {
-        // RFC4408 の ABNF がダメダメなので, 前からパースすると macro-string が domain-end を喰っちゃう.
-        // よって文法的に正しくても domain-end にはマッチしない場合もある.
-        parse_stat = SIDF_STAT_OK;
-        *nextp = p;
-    } else {
+    // 前からパースすると macro-string が domain-end を喰っちゃうので,
+    // macro-string が macro-literal で終端している場合のみ,
+    // domain-end が toplabel で終端しているか確認する.
+    const char *q;
+    if (literal_terminated && 0 == SidfMacro_skipbackTopLabel(head, tail, &q)) {
+        SidfLogPermFail(request->policy,
+                        "domain-spec does not terminate with domain-end: domain-spec=%.*s",
+                        (int) (tail - head), head);
+
         *nextp = head;
+        return SIDF_STAT_RECORD_NOT_MATCH;
     }   // end if
-    return parse_stat;
-}   // end function : SidfMacro_parseDomainSpec
+
+    *nextp = p;
+    return SIDF_STAT_OK;
+}   // end function: SidfMacro_parseDomainSpec
