@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2007-2011 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: sidfrecord.c 841 2009-03-27 08:59:56Z takahiko $
+ * $Id: sidfrecord.c 1346 2011-08-10 02:15:43Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: sidfrecord.c 841 2009-03-27 08:59:56Z takahiko $");
+RCSID("$Id: sidfrecord.c 1346 2011-08-10 02:15:43Z takahiko $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,11 +36,11 @@ RCSID("$Id: sidfrecord.c 841 2009-03-27 08:59:56Z takahiko $");
 #define SIDF_RECORD_SPF1_PREFIX "v=spf1"
 #define SIDF_RECORD_SIDF20_PREFIX "spf2.0"
 
-// ip4-cidr-length の最大値
+// maximum value of ip4-cidr-length
 #define SIDF_IP4_MAX_CIDR_LENGTH 32
-// ip6-cidr-length の最大値
+// maximum value of ip6-cidr-length
 #define SIDF_IP6_MAX_CIDR_LENGTH 128
-// cidr-length を表記するのに必要な最大文字数
+// the number of digits to represent cidr-length decimally
 // 128 が最大値なので3桁あれば十分
 #define SIDF_RECORD_CIDRLEN_MAX_WIDTH 3
 #define SIDF_MACRO_EXPANSION_MAX_LENGTH 253
@@ -168,12 +168,12 @@ SidfRecord_lookupSidfScope(const char *head, const char *tail, const char **next
     };
 
     if (0 < XSkip_spfName(head, tail, nextp)) {
-        return KeywordMap_lookupByStringSlice(sidf_scope_table, head, *nextp);
+        return KeywordMap_lookupByCaseStringSlice(sidf_scope_table, head, *nextp);
     } else {
         *nextp = head;
         return SIDF_RECORD_SCOPE_NULL;
     }   // end if
-}   // end function : SidfRecord_lookupSidfScope
+}   // end function: SidfRecord_lookupSidfScope
 
 /*
  * [RFC4406]
@@ -188,14 +188,14 @@ SidfRecord_parseVersion(const SidfRequest *request, const char *head, const char
                         const char **nextp, SidfRecordScope *scope)
 {
     // SPF レコードかチェック
-    if (0 < XSkip_string(head, tail, SIDF_RECORD_SPF1_PREFIX, nextp)) {
+    if (0 < XSkip_casestring(head, tail, SIDF_RECORD_SPF1_PREFIX, nextp)) {
         *scope = SIDF_RECORD_SCOPE_SPF1;
         return SIDF_STAT_OK;
     }   // end if
 
     // SIDF レコードかチェック
     const char *p;
-    if (0 < XSkip_string(head, tail, SIDF_RECORD_SIDF20_PREFIX, &p)
+    if (0 < XSkip_casestring(head, tail, SIDF_RECORD_SIDF20_PREFIX, &p)
         && 0 < XSkip_char(p, tail, '/', &p)) {
         SidfRecordScope record_scope = 0;
         const char *scope_tail;
@@ -228,7 +228,7 @@ SidfRecord_parseVersion(const SidfRequest *request, const char *head, const char
     *nextp = head;
     *scope = SIDF_RECORD_SCOPE_NULL;
     return SIDF_STAT_RECORD_SYNTAX_VIOLATION;
-}   // end function : SidfRecord_parseVersion
+}   // end function: SidfRecord_parseVersion
 
 static SidfQualifier
 SidfRecord_parseQualifier(const char *head, const char *tail, const char **nextp)
@@ -248,7 +248,7 @@ SidfRecord_parseQualifier(const char *head, const char *tail, const char **nextp
 
     *nextp = head;
     return p->qualifier;
-}   // end function : SidfRecord_parseQualifier
+}   // end function: SidfRecord_parseQualifier
 
 static SidfStat
 SidfRecord_parseDomainSpec(SidfRecord *self, const char *head, const char *tail, SidfTerm *term,
@@ -302,7 +302,7 @@ SidfRecord_parseDomainSpec(SidfRecord *self, const char *head, const char *tail,
         }   // end if
     }   // end if
     return parse_stat;
-}   // end function : SidfRecord_parseDomainSpec
+}   // end function: SidfRecord_parseDomainSpec
 
 static SidfStat
 SidfRecord_parseIp4Addr(const char *head, const char *tail, SidfTerm *term, const char **nextp)
@@ -317,7 +317,7 @@ SidfRecord_parseIp4Addr(const char *head, const char *tail, SidfTerm *term, cons
         *nextp = head;
         return SIDF_STAT_RECORD_SYNTAX_VIOLATION;
     }   // end if
-}   // end function : SidfRecord_parseIp4Addr
+}   // end function: SidfRecord_parseIp4Addr
 
 static SidfStat
 SidfRecord_parseIp6Addr(const char *head, const char *tail, SidfTerm *term, const char **nextp)
@@ -332,7 +332,7 @@ SidfRecord_parseIp6Addr(const char *head, const char *tail, SidfTerm *term, cons
         *nextp = head;
         return SIDF_STAT_RECORD_SYNTAX_VIOLATION;
     }   // end if
-}   // end function : SidfRecord_parseIp6Addr
+}   // end function: SidfRecord_parseIp6Addr
 
 static SidfStat
 SidfRecord_parsebackCidrLength(const char *head, const char *tail,
@@ -355,7 +355,7 @@ SidfRecord_parsebackCidrLength(const char *head, const char *tail,
         *cidrlength = 0;
         return SIDF_STAT_RECORD_NOT_MATCH;
     }   // end if
-}   // end function : SidfRecord_parsebackCidrLength
+}   // end function: SidfRecord_parsebackCidrLength
 
 /**
  * @return SIDF_STAT_OK: maxcidrlen 以下の cidr-length を取得した.
@@ -382,7 +382,7 @@ SidfRecord_parsebackSingleCidrLength(const SidfPolicy *policy, const char *head,
     default:
         abort();
     }   // end switch
-}   // end function : SidfRecord_parsebackSingleCidrLength
+}   // end function: SidfRecord_parsebackSingleCidrLength
 
 /**
  * @return SIDF_STAT_OK: maxcidrlen 以下の cidr-length を取得した.
@@ -400,7 +400,7 @@ SidfRecord_parsebackIp4CidrLength(const SidfPolicy *policy, const char *head, co
                                                                &cidrlength);
     term->ip4cidr = (SIDF_STAT_OK == parse_stat) ? cidrlength : SIDF_IP4_MAX_CIDR_LENGTH;
     return parse_stat;
-}   // end function : SidfRecord_parsebackIp4CidrLength
+}   // end function: SidfRecord_parsebackIp4CidrLength
 
 /**
  * @return SIDF_STAT_OK: maxcidrlen 以下の cidr-length を取得した.
@@ -418,7 +418,7 @@ SidfRecord_parsebackIp6CidrLength(const SidfPolicy *policy, const char *head, co
                                                                &cidrlength);
     term->ip6cidr = (SIDF_STAT_OK == parse_stat) ? cidrlength : SIDF_IP6_MAX_CIDR_LENGTH;
     return parse_stat;
-}   // end function : SidfRecord_parsebackIp6CidrLength
+}   // end function: SidfRecord_parsebackIp6CidrLength
 
 /**
  * @return SIDF_STAT_OK: maxcidrlen 以下の cidr-length を取得した.
@@ -469,7 +469,7 @@ SidfRecord_parsebackDualCidrLength(const SidfPolicy *policy, const char *head, c
         abort();
     }   // end switch
     return parse_stat;
-}   // end function : SidfRecord_parsebackDualCidrLength
+}   // end function: SidfRecord_parsebackDualCidrLength
 
 /**
  * @return SIDF_STAT_OK: maxcidrlen 以下の cidr-length を取得した.
@@ -493,7 +493,7 @@ SidfRecord_parseCidrLength(const SidfPolicy *policy, SidfTermCidrOption cidr_typ
     default:
         abort();
     }   // end switch
-}   // end function : SidfRecord_parseCidrLength
+}   // end function: SidfRecord_parseCidrLength
 
 static SidfStat
 SidfRecord_parseTermTargetName(SidfRecord *self, SidfTermParamType param_type, const char *head,
@@ -512,7 +512,7 @@ SidfRecord_parseTermTargetName(SidfRecord *self, SidfTermParamType param_type, c
     default:
         abort();
     }   // end switch
-}   // end function : SidfRecord_parseTermTargetName
+}   // end function: SidfRecord_parseTermTargetName
 
 static SidfTerm *
 SidfTerm_new(SidfTermParamType param_type)
@@ -540,7 +540,7 @@ SidfTerm_new(SidfTermParamType param_type)
     }   // end if
     memset(self, 0, sizeof(SidfTerm) + contentsize);
     return self;
-}   // end function : SidfTerm_new
+}   // end function: SidfTerm_new
 
 static void
 SidfTerm_free(SidfTerm *self)
@@ -550,7 +550,7 @@ SidfTerm_free(SidfTerm *self)
         free(self->param.domain);
     }   // end if
     free(self);
-}   // end function : SidfTerm_free
+}   // end function: SidfTerm_free
 
 static const SidfTermAttribute *
 SidfRecord_lookupMechanismAttribute(const char *head, const char *tail)
@@ -568,7 +568,7 @@ SidfRecord_lookupMechanismAttribute(const char *head, const char *tail)
         }   // end if
     }   // end for
     return NULL;
-}   // end function : SidfRecord_lookupMechanismAttribute
+}   // end function: SidfRecord_lookupMechanismAttribute
 
 static const SidfTermAttribute *
 SidfRecord_lookupModifierAttribute(const char *head, const char *tail)
@@ -586,7 +586,7 @@ SidfRecord_lookupModifierAttribute(const char *head, const char *tail)
         }   // end if
     }   // end for
     return NULL;
-}   // end function : SidfRecord_lookupModifierAttribute
+}   // end function: SidfRecord_lookupModifierAttribute
 
 /**
  * @param head メカニズムの直後を指すポインタ
@@ -712,7 +712,7 @@ SidfRecord_buildTerm(SidfRecord *self, const char *head, const char *tail,
     }   // end if
 
     return SIDF_STAT_OK;
-}   // end function : SidfRecord_parseTermParam
+}   // end function: SidfRecord_parseTermParam
 
 static SidfStat
 SidfRecord_parse(SidfRecord *self, const char *head, const char *tail)
@@ -784,13 +784,17 @@ SidfRecord_parse(SidfRecord *self, const char *head, const char *tail)
     if (term_head == tail) {
         return SIDF_STAT_OK;
     } else {
-        // レコードのパースを中断した
+        // abort parsing the record
         SidfLogPermFail(self->request->policy, "unparsable term: domain=%s, near=[%.*s]",
                         self->domain, (int) (tail - term_head), term_head);
         return SIDF_STAT_RECORD_SYNTAX_VIOLATION;
     }   // end if
-}   // end function : SidfRecord_parseTerms
+}   // end function: SidfRecord_parseTerms
 
+/**
+ * release SidfRecord object
+ * @param self SidfRecord object to release
+ */
 void
 SidfRecord_free(SidfRecord *self)
 {
@@ -805,8 +809,12 @@ SidfRecord_free(SidfRecord *self)
         SidfTerm_free(self->modifiers.exp);
     }   // end if
     free(self);
-}   // end function : SidfRecord_free
+}   // end function: SidfRecord_free
 
+/**
+ * create SidfRecord object
+ * @return initialized SidfRecord object, or NULL if memory allocation failed.
+ */
 static SidfRecord *
 SidfRecord_new(const SidfRequest *request)
 {
@@ -828,7 +836,7 @@ SidfRecord_new(const SidfRequest *request)
   cleanup:
     SidfRecord_free(self);
     return NULL;
-}   // end function : SidfRecord_new
+}   // end function: SidfRecord_new
 
 /**
  * SPFレコードのスコープを除いた部分をパースして, SidfRecord オブジェクトを構築する.
@@ -863,7 +871,7 @@ SidfRecord_build(const SidfRequest *request, SidfRecordScope scope, const char *
         SidfRecord_free(self);
     }   // end if
     return build_stat;
-}   // end function : SidfRecord_build
+}   // end function: SidfRecord_build
 
 /**
  * 指定した SPF/SIDF レコードのスコープを取得する.
@@ -887,4 +895,4 @@ SidfRecord_getSidfScope(const SidfRequest *request, const char *record_head,
         *scope = SIDF_RECORD_SCOPE_NULL;
         return SIDF_STAT_RECORD_SYNTAX_VIOLATION;
     }   //end if
-}   // end function : SidfRecord_getSidfScope
+}   // end function: SidfRecord_getSidfScope
