@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2007-2011 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: sidfrequest.c 751 2009-03-15 17:06:07Z takahiko $
+ * $Id: sidfrequest.c 1343 2011-07-30 19:21:50Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: sidfrequest.c 751 2009-03-15 17:06:07Z takahiko $");
+RCSID("$Id: sidfrequest.c 1343 2011-07-30 19:21:50Z takahiko $");
 
 #include <stdio.h>
 #include <string.h>
@@ -22,6 +22,7 @@ RCSID("$Id: sidfrequest.c 751 2009-03-15 17:06:07Z takahiko $");
 #include <netdb.h>
 #include <resolv.h>
 
+#include "stdaux.h"
 #include "ptrop.h"
 #include "sidflogger.h"
 #include "strarray.h"
@@ -36,10 +37,6 @@ RCSID("$Id: sidfrequest.c 751 2009-03-15 17:06:07Z takahiko $");
 #include "sidfrequest.h"
 #include "sidfmacro.h"
 
-#ifndef MIN
-#define MIN(a,b) (((a)<(b))?(a):(b))
-#endif
-
 #define SIDF_REQUEST_DEFAULT_LOCALPART "postmaster"
 
 typedef struct SidfRawRecord {
@@ -55,7 +52,7 @@ static unsigned int
 SidfRequest_getDepth(const SidfRequest *self)
 {
     return self->redirect_depth + self->include_depth;
-}   // end function : SidfRequest_getDepth
+}   // end function: SidfRequest_getDepth
 
 static SidfStat
 SidfRequest_pushDomain(SidfRequest *self, const char *domain)
@@ -66,39 +63,39 @@ SidfRequest_pushDomain(SidfRequest *self, const char *domain)
         SidfLogNoResource(self->policy);
         return SIDF_STAT_NO_RESOURCE;
     }   // end if
-}   // end function : SidfRequest_pushDomain
+}   // end function: SidfRequest_pushDomain
 
 static void
 SidfRequest_popDomain(SidfRequest *self)
 {
     StrArray_unappend(self->domain);
-}   // end function : SidfRequest_popDomain
+}   // end function: SidfRequest_popDomain
 
 const char *
 SidfRequest_getDomain(const SidfRequest *self)
 {
     size_t n = StrArray_getCount(self->domain);
     return 0 < n ? StrArray_get(self->domain, n - 1) : NULL;
-}   // end function : SidfRequest_getDomain
+}   // end function: SidfRequest_getDomain
 
 static SidfScore
 SidfRequest_getScoreByQualifier(SidfQualifier qualifier)
 {
     // SidfQualifier は各スコアに対応する値を持たせているのでキャストするだけでよい
     return (SidfScore) qualifier;
-}   // end function : SidfRequest_getScoreByQualifier
+}   // end function: SidfRequest_getScoreByQualifier
 
 bool
 SidfRequest_isSenderContext(const SidfRequest *self)
 {
     return self->is_sender_context;
-}   // end function : SidfRequest_isSenderContext
+}   // end function: SidfRequest_isSenderContext
 
 const char *
 SidfRequest_getExplanation(const SidfRequest *self)
 {
     return self->explanation;
-}   // end function : SidfRequest_getExplanation
+}   // end function: SidfRequest_getExplanation
 
 static SidfStat
 SidfRequest_setExplanation(SidfRequest *self, const char *domain, const char *exp_macro)
@@ -124,7 +121,7 @@ SidfRequest_setExplanation(SidfRequest *self, const char *domain, const char *ex
                     exp_macro);
     }   // end if
     return parse_stat;
-}   // end function : SidfRequest_setExplanation
+}   // end function: SidfRequest_setExplanation
 
 /**
  * スコープに一致する唯一つのレコードを選択する.
@@ -149,7 +146,7 @@ SidfRequest_uniqueByScope(const SidfRawRecord *rawrecords, unsigned int recordnu
     }   // end for
 
     return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_uniqueByScope
+}   // end function: SidfRequest_uniqueByScope
 
 /**
  * @return 成功した場合は SIDF_SCORE_NULL, SPFレコード取得の際にエラーが発生した場合は SIDF_SCORE_NULL 以外.
@@ -210,6 +207,11 @@ SidfRequest_fetch(const SidfRequest *self, const char *domain, DnsTxtResponse **
              * check_host() exits immediately with the result "TempError".
              */
             return SIDF_SCORE_TEMPERROR;
+        case DNS_STAT_BADREQUEST:
+        case DNS_STAT_SYSTEM:
+        case DNS_STAT_RESOLVER:
+        case DNS_STAT_RESOLVER_INTERNAL:
+        case DNS_STAT_NOMEMORY:
         default:
             return SIDF_SCORE_SYSERROR;
         }   // end switch
@@ -266,10 +268,15 @@ SidfRequest_fetch(const SidfRequest *self, const char *domain, DnsTxtResponse **
          * check_host() exits immediately with the result "TempError".
          */
         return SIDF_SCORE_TEMPERROR;
+    case DNS_STAT_BADREQUEST:
+    case DNS_STAT_SYSTEM:
+    case DNS_STAT_RESOLVER:
+    case DNS_STAT_RESOLVER_INTERNAL:
+    case DNS_STAT_NOMEMORY:
     default:
         return SIDF_SCORE_SYSERROR;
     }   // end switch
-}   // end function : SidfRequest_fetch
+}   // end function: SidfRequest_fetch
 
 static SidfScore
 SidfRequest_lookupRecord(const SidfRequest *self, const char *domain, SidfRecord **record)
@@ -344,13 +351,13 @@ SidfRequest_lookupRecord(const SidfRequest *self, const char *domain, SidfRecord
     default:
         return SIDF_SCORE_PERMERROR;
     }   // end switch
-}   // end function : SidfRequest_lookupRecord
+}   // end function: SidfRequest_lookupRecord
 
 static const char *
 SidfRequest_getTargetName(const SidfRequest *self, const SidfTerm *term)
 {
     return term->querydomain ? term->querydomain : SidfRequest_getDomain(self);
-}   // end function : SidfRequest_getTargetName
+}   // end function: SidfRequest_getTargetName
 
 /*
  * メカニズム評価中の DNS レスポンスエラーコードを SIDF のスコアにマップする.
@@ -387,10 +394,15 @@ SidfRequest_mapMechDnsResponseToSidfScore(dns_stat_t resolv_stat)
     case DNS_STAT_RESERVED14:
     case DNS_STAT_RESERVED15:
         return SIDF_SCORE_TEMPERROR;
+    case DNS_STAT_BADREQUEST:
+    case DNS_STAT_SYSTEM:
+    case DNS_STAT_RESOLVER:
+    case DNS_STAT_RESOLVER_INTERNAL:
+    case DNS_STAT_NOMEMORY:
     default:
         return SIDF_SCORE_SYSERROR;
     }   // end switch
-}   // end function : SidfRequest_mapMechDnsResponseToSidfScore
+}   // end function: SidfRequest_mapMechDnsResponseToSidfScore
 
 static SidfScore
 SidfRequest_incrementDnsMechCounter(SidfRequest *self)
@@ -404,7 +416,7 @@ SidfRequest_incrementDnsMechCounter(SidfRequest *self)
                         SidfRequest_getDomain(self));
         return SIDF_SCORE_PERMERROR;
     }   // end if
-}   // end function : SidfRequest_incrementDnsMechCounter
+}   // end function: SidfRequest_incrementDnsMechCounter
 
 static SidfScore
 SidfRequest_checkMaliceOfCidrLength(const SidfRequest *self, char ip_version,
@@ -436,7 +448,7 @@ SidfRequest_checkMaliceOfCidrLength(const SidfRequest *self, char ip_version,
         }   // end switch
     }   // end if
     return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_checkMaliceOfCidrLength
+}   // end function: SidfRequest_checkMaliceOfCidrLength
 
 static SidfScore
 SidfRequest_checkMaliceOfIp4CidrLength(const SidfRequest *self, const SidfTerm *term)
@@ -444,7 +456,7 @@ SidfRequest_checkMaliceOfIp4CidrLength(const SidfRequest *self, const SidfTerm *
     return SidfRequest_checkMaliceOfCidrLength(self, '4', term->ip4cidr,
                                                self->policy->malicious_ip4_cidr_length,
                                                self->policy->action_on_malicious_ip4_cidr_length);
-}   // end function : SidfRequest_checkMaliceOfIp4CidrLength
+}   // end function: SidfRequest_checkMaliceOfIp4CidrLength
 
 static SidfScore
 SidfRequest_checkMaliceOfIp6CidrLength(const SidfRequest *self, const SidfTerm *term)
@@ -452,7 +464,7 @@ SidfRequest_checkMaliceOfIp6CidrLength(const SidfRequest *self, const SidfTerm *
     return SidfRequest_checkMaliceOfCidrLength(self, '6', term->ip6cidr,
                                                self->policy->malicious_ip6_cidr_length,
                                                self->policy->action_on_malicious_ip6_cidr_length);
-}   // end function : SidfRequest_checkMaliceOfIp6CidrLength
+}   // end function: SidfRequest_checkMaliceOfIp6CidrLength
 
 static SidfScore
 SidfRequest_checkMaliceOfDualCidrLength(const SidfRequest *self, const SidfTerm *term)
@@ -462,7 +474,7 @@ SidfRequest_checkMaliceOfDualCidrLength(const SidfRequest *self, const SidfTerm
         return score;
     }   // end if
     return SidfRequest_checkMaliceOfIp6CidrLength(self, term);
-}   // end function : SidfRequest_checkMaliceOfDualCidrLength
+}   // end function: SidfRequest_checkMaliceOfDualCidrLength
 
 static SidfScore
 SidfRequest_checkPlusAllDirective(const SidfRequest *self, const SidfTerm *term)
@@ -490,7 +502,7 @@ SidfRequest_checkPlusAllDirective(const SidfRequest *self, const SidfTerm *term)
         }   // end switch
     }   // end if
     return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_checkPlusAllDirective
+}   // end function: SidfRequest_checkPlusAllDirective
 
 static SidfScore
 SidfRequest_evalMechAll(const SidfRequest *self, const SidfTerm *term)
@@ -503,7 +515,7 @@ SidfRequest_evalMechAll(const SidfRequest *self, const SidfTerm *term)
     return SIDF_SCORE_NULL == self->policy->overwrite_all_directive_score
         ? SidfRequest_getScoreByQualifier(term->qualifier)
         : self->policy->overwrite_all_directive_score;
-}   // end function : SidfRequest_evalMechAll
+}   // end function: SidfRequest_evalMechAll
 
 static SidfScore
 SidfRequest_evalMechInclude(SidfRequest *self, const SidfTerm *term)
@@ -555,7 +567,7 @@ SidfRequest_evalMechInclude(SidfRequest *self, const SidfTerm *term)
     default:
         abort();
     }   // end switch
-}   // end function : SidfRequest_evalMechInclude
+}   // end function: SidfRequest_evalMechInclude
 
 /*
  * "a" メカニズムと "mx" メカニズムの共通部分を実装する関数
@@ -607,7 +619,7 @@ SidfRequest_evalByALookup(SidfRequest *self, const char *domain, const SidfTerm
     }   // end if
 
     return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalByALookup
+}   // end function: SidfRequest_evalByALookup
 
 static SidfScore
 SidfRequest_evalMechA(SidfRequest *self, const SidfTerm *term)
@@ -621,7 +633,7 @@ SidfRequest_evalMechA(SidfRequest *self, const SidfTerm *term)
 
     const char *domain = SidfRequest_getTargetName(self, term);
     return SidfRequest_evalByALookup(self, domain, term);
-}   // end function : SidfRequest_evalMechA
+}   // end function: SidfRequest_evalMechA
 
 static SidfScore
 SidfRequest_evalMechMx(SidfRequest *self, const SidfTerm *term)
@@ -660,70 +672,82 @@ SidfRequest_evalMechMx(SidfRequest *self, const SidfTerm *term)
     }   // end for
     DnsMxResponse_free(respmx);
     return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalMechMx
+}   // end function: SidfRequest_evalMechMx
 
-static SidfScore
-SidfRequest_evalMechPtrValidate4(SidfRequest *self, const SidfTerm *term, const char *revdomain)
+/**
+ * @param request SidfRequest object.
+ * @param revdomain
+ * @return 1 if IP addresses match.
+ *         0 if IP addresses doesn't match.
+ *         -1 if DNS error occurred.
+ */
+static int
+SidfRequest_isValidatedDomainName4(const SidfRequest *self, const char *revdomain)
 {
     DnsAResponse *resp;
     dns_stat_t query_stat = DnsResolver_lookupA(self->resolver, revdomain, &resp);
     if (DNS_STAT_NOERROR != query_stat) {
-        /*
-         * "ptr" メカニズムの評価中に A レコードのルックアップでエラーが発生した場合,
-         * そのドメイン名をスキップして評価を続行する.
-         * [RFC4408] 5.5.
-         * If a DNS error occurs while doing the PTR RR lookup, then this
-         * mechanism fails to match.  If a DNS error occurs while doing an A RR
-         * lookup, then that domain name is skipped and the search continues.
-         */
-        SidfLogDnsError(self->policy, "DNS lookup failure (ignored): rrtype=a, domain=%s, err=%s",
+        SidfLogDnsError(self->policy, "DNS lookup failure: rrtype=a, domain=%s, err=%s",
                         revdomain, DnsResolver_getErrorString(self->resolver));
-        return SIDF_SCORE_NULL;
+        return -1;
     }   // end if
     for (size_t m = 0; m < DnsAResponse_size(resp); ++m) {
-        /*
-         * revdomain が domain で終わっていれば match
-         * [RFC4408] 5.5.
-         * Check all validated domain names to see if they end in the
-         * <target-name> domain.  If any do, this mechanism matches.  If no
-         * validated domain name can be found, or if none of the validated
-         * domain names end in the <target-name>, this mechanism fails to match.
-         * (snip)
-         * This mechanism matches if the <target-name> is either an ancestor of
-         * a validated domain name or if the <target-name> and a validated
-         * domain name are the same.  For example: "mail.example.com" is within
-         * the domain "example.com", but "mail.bad-example.com" is not.
-         */
         if (0 == memcmp(DnsAResponse_addr(resp, m), &(self->ipaddr.addr4), NS_INADDRSZ)) {
             DnsAResponse_free(resp);
-            return SidfRequest_getScoreByQualifier(term->qualifier);
+            return 1;
         }   // end if
     }   // end for
     DnsAResponse_free(resp);
-    return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalMechPtrValidate4
+    return 0;
+}   // end function: SidfRequest_isValidatedDomainName4
 
-static SidfScore
-SidfRequest_evalMechPtrValidate6(SidfRequest *self, const SidfTerm *term, const char *revdomain)
+/**
+ * @param request SidfRequest object.
+ * @param revdomain
+ * @return 1 if IP addresses match.
+ *         0 if IP addresses doesn't match.
+ *         -1 if DNS error occurred.
+ */
+static int
+SidfRequest_isValidatedDomainName6(const SidfRequest *self, const char *revdomain)
 {
-    // ロジックに関する詳細は SidfRequest_evalMechPtrValidate4() のコメントと重複するので省略
     DnsAaaaResponse *resp;
     dns_stat_t query_stat = DnsResolver_lookupAaaa(self->resolver, revdomain, &resp);
     if (DNS_STAT_NOERROR != query_stat) {
         SidfLogDnsError(self->policy,
                         "DNS lookup failure (ignored): rrtype=aaaa, domain=%s, err=%s", revdomain,
                         DnsResolver_getErrorString(self->resolver));
-        return SIDF_SCORE_NULL;
+        return -1;
     }   // end if
     for (size_t m = 0; m < DnsAaaaResponse_size(resp); ++m) {
         if (0 == memcmp(DnsAaaaResponse_addr(resp, m), &(self->ipaddr.addr6), NS_IN6ADDRSZ)) {
             DnsAaaaResponse_free(resp);
-            return SidfRequest_getScoreByQualifier(term->qualifier);
+            return 1;
         }   // end if
     }   // end for
     DnsAaaaResponse_free(resp);
-    return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalMechPtrValidate6
+    return 0;
+}   // end function: SidfMacro_isValidatedDomainName6
+
+/*
+ * @param request SidfRequest object.
+ * @param revdomain
+ * @return 1 if IP addresses match.
+ *         0 if IP addresses doesn't match.
+ *         -1 if DNS error occurred.
+ */
+int
+SidfRequest_isValidatedDomainName(const SidfRequest *self, const char *revdomain)
+{
+    switch (self->sa_family) {
+    case AF_INET:
+        return SidfRequest_isValidatedDomainName4(self, revdomain);
+    case AF_INET6:
+        return SidfRequest_isValidatedDomainName6(self, revdomain);
+    default:
+        abort();
+    }   // end switch
+}   // end function: SidfRequest_isValidatedDomainName
 
 static SidfScore
 SidfRequest_evalMechPtr(SidfRequest *self, const SidfTerm *term)
@@ -735,12 +759,9 @@ SidfRequest_evalMechPtr(SidfRequest *self, const SidfTerm *term)
         DnsResolver_lookupPtr(self->resolver, self->sa_family, &(self->ipaddr), &respptr);
     if (DNS_STAT_NOERROR != ptrquery_stat) {
         /*
-         * "ptr" メカニズムの評価中に PTR レコードのルックアップでエラーが発生した場合,
-         * このメカニズムにマッチしなかったという扱いになる.
          * [RFC4408] 5.5.
          * If a DNS error occurs while doing the PTR RR lookup, then this
-         * mechanism fails to match.  If a DNS error occurs while doing an A RR
-         * lookup, then that domain name is skipped and the search continues.
+         * mechanism fails to match.
          */
         char addrbuf[INET6_ADDRSTRLEN];
         (void) inet_ntop(self->sa_family, &(self->ipaddr), addrbuf, sizeof(addrbuf));
@@ -760,33 +781,36 @@ SidfRequest_evalMechPtr(SidfRequest *self, const SidfTerm *term)
      * a "ptr" mechanism (see Section 10).  If <ip> is among the returned IP
      * addresses, then that domain name is validated.
      */
-    for (size_t n = 0; n < MIN(DnsPtrResponse_size(respptr), self->policy->max_ptrrr_per_ptrmech);
-         ++n) {
+    size_t resp_num_limit = MIN(DnsPtrResponse_size(respptr), self->policy->max_ptrrr_per_ptrmech);
+    for (size_t n = 0; n < resp_num_limit; ++n) {
         // アルゴリズムをよく読むと validated domain が <target-name> で終わっているかどうかの判断を
         // 先におこなった方が DNS ルックアップの回数が少なくて済む場合があることがわかる.
+        /*
+         * [RFC4408] 5.5.
+         * Check all validated domain names to see if they end in the
+         * <target-name> domain.  If any do, this mechanism matches.  If no
+         * validated domain name can be found, or if none of the validated
+         * domain names end in the <target-name>, this mechanism fails to match.
+         */
         if (!InetDomain_isParent(domain, DnsPtrResponse_domain(respptr, n))) {
             continue;
         }   // end if
 
-        SidfScore score;
-        switch (self->sa_family) {
-        case AF_INET:
-            score = SidfRequest_evalMechPtrValidate4(self, term, DnsPtrResponse_domain(respptr, n));
-            break;
-        case AF_INET6:
-            score = SidfRequest_evalMechPtrValidate6(self, term, DnsPtrResponse_domain(respptr, n));
-            break;
-        default:
-            abort();
-        }   // end switch
-        if (SIDF_SCORE_NULL != score) {
+        int validation_stat =
+            SidfRequest_isValidatedDomainName(self, DnsPtrResponse_domain(respptr, n));
+        /*
+         * [RFC4408] 5.5.
+         * If a DNS error occurs while doing an A RR
+         * lookup, then that domain name is skipped and the search continues.
+         */
+        if (1 == validation_stat) {
             DnsPtrResponse_free(respptr);
-            return score;
+            return SidfRequest_getScoreByQualifier(term->qualifier);
         }   // end if
     }   // end for
     DnsPtrResponse_free(respptr);
     return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalMechPtr
+}   // end function: SidfRequest_evalMechPtr
 
 static SidfScore
 SidfRequest_evalMechIp4(const SidfRequest *self, const SidfTerm *term)
@@ -799,7 +823,7 @@ SidfRequest_evalMechIp4(const SidfRequest *self, const SidfTerm *term)
     return (AF_INET == self->sa_family
             && 0 == bitmemcmp(&(self->ipaddr.addr4), &(term->param.addr4), term->ip4cidr))
         ? SidfRequest_getScoreByQualifier(term->qualifier) : SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalMechIp4
+}   // end function: SidfRequest_evalMechIp4
 
 static SidfScore
 SidfRequest_evalMechIp6(const SidfRequest *self, const SidfTerm *term)
@@ -812,7 +836,7 @@ SidfRequest_evalMechIp6(const SidfRequest *self, const SidfTerm *term)
     return (AF_INET6 == self->sa_family
             && 0 == bitmemcmp(&(self->ipaddr.addr6), &(term->param.addr6), term->ip6cidr))
         ? SidfRequest_getScoreByQualifier(term->qualifier) : SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalMechIp6
+}   // end function: SidfRequest_evalMechIp6
 
 static SidfScore
 SidfRequest_evalMechExists(SidfRequest *self, const SidfTerm *term)
@@ -829,7 +853,7 @@ SidfRequest_evalMechExists(SidfRequest *self, const SidfTerm *term)
     size_t num = DnsAResponse_size(resp);
     DnsAResponse_free(resp);
     return (0 < num) ? SidfRequest_getScoreByQualifier(term->qualifier) : SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalMechExists
+}   // end function: SidfRequest_evalMechExists
 
 static SidfScore
 SidfRequest_evalModRedirect(SidfRequest *self, const SidfTerm *term)
@@ -850,7 +874,7 @@ SidfRequest_evalModRedirect(SidfRequest *self, const SidfTerm *term)
      * is a "PermError" rather than "None".
      */
     return SIDF_SCORE_NONE == eval_score ? SIDF_SCORE_PERMERROR : eval_score;
-}   // end function : SidfRequest_evalModRedirect
+}   // end function: SidfRequest_evalModRedirect
 
 static SidfStat
 SidfRequest_evalModExplanation(SidfRequest *self, const SidfTerm *term)
@@ -882,7 +906,7 @@ SidfRequest_evalModExplanation(SidfRequest *self, const SidfTerm *term)
         SidfRequest_setExplanation(self, term->querydomain, DnsTxtResponse_data(resp, 0));
     DnsTxtResponse_free(resp);
     return expand_stat;
-}   // end function : SidfRequest_evalModExplanation
+}   // end function: SidfRequest_evalModExplanation
 
 static SidfScore
 SidfRequest_evalMechanism(SidfRequest *self, const SidfTerm *term)
@@ -917,7 +941,7 @@ SidfRequest_evalMechanism(SidfRequest *self, const SidfTerm *term)
     default:
         abort();
     }   // end switch
-}   // end function : SidfRequest_evalMechanism
+}   // end function: SidfRequest_evalMechanism
 
 static SidfScore
 SidfRequest_checkDomain(const SidfRequest *self, const char *domain)
@@ -931,22 +955,33 @@ SidfRequest_checkDomain(const SidfRequest *self, const char *domain)
      * name, or if the DNS lookup returns "domain does not exist" (RCODE 3),
      * check_host() immediately returns the result "None".
      */
-    size_t domain_len = strlen(domain);
-    if (self->policy->max_domain_len < domain_len) {
-        SidfLogPermFail(self->policy,
-                        "<domain> length for check_host too long: length=%u, domain(50)=%.50s",
-                        (unsigned int) domain_len, domain);
-        return SIDF_SCORE_NONE;
-    }   // end if
-
-    // 文字種のチェック. 2821-Domain だとキツいのでちょっと緩め.
-    const char *p;
-    const char *domain_tail = domain + domain_len;
-    XSkip_dotAtomText(domain, domain_tail, &p);
-    XSkip_char(p, domain_tail, '.', &p);
+    const char *p = domain;
+    const char *domain_tail = STRTAIL(domain);
+    while (p < domain_tail) {
+        // 同時に文字種のチェック. 2821-Domain だとキツいのでちょっと緩め.
+        int label_len = XSkip_atextBlock(p, domain_tail, &p);
+        if (label_len <= 0) {
+            break;
+        } else if ((int) self->policy->max_label_len < label_len) {
+            SidfLogPermFail(self->policy,
+                            "label length of <domain> argument of check_host exceeds its limit: length=%u, limit=%u, domain(256)=%.256",
+                            (unsigned int) label_len, self->policy->max_label_len, domain);
+            return SIDF_SCORE_NONE;
+        }   // end if
+        if (0 >= XSkip_char(p, domain_tail, '.', &p)) {
+            /*
+             * <domain-spec> may end with '.' (dot, 0x2e)
+             * [RFC4408] 8.1.
+             * domain-spec      = macro-string domain-end
+             * domain-end       = ( "." toplabel [ "." ] ) / macro-expand
+             */
+            break;
+        }   // end if
+    }   // end while
     if (domain_tail != p) {
         SidfLogPermFail(self->policy,
-                        "<domain> for check_host doesn't match domain-name: domain=%s", domain);
+                        "<domain> argument of check_host doesn't match domain-name: domain=%s",
+                        domain);
         return SIDF_SCORE_NONE;
     }   // end if
 
@@ -957,7 +992,7 @@ SidfRequest_checkDomain(const SidfRequest *self, const char *domain)
     }   // end if
 
     return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_checkDomain
+}   // end function: SidfRequest_checkDomain
 
 static SidfScore
 SidfRequest_evalDirectives(SidfRequest *self, const PtrArray *directives)
@@ -976,7 +1011,7 @@ SidfRequest_evalDirectives(SidfRequest *self, const PtrArray *directives)
                      domain, i, term->attr->name);
     }   // end if
     return SIDF_SCORE_NULL;
-}   // end function : SidfRequest_evalDirectives
+}   // end function: SidfRequest_evalDirectives
 
 static SidfScore
 SidfRequest_evalLocalPolicy(SidfRequest *self)
@@ -1017,8 +1052,13 @@ SidfRequest_evalLocalPolicy(SidfRequest *self)
                      SidfEnum_lookupScoreByValue(local_policy_score));
         return local_policy_score;
     }   // end switch
-}   // end function : SidfRequest_evalLocalPolicy
+}   // end function: SidfRequest_evalLocalPolicy
 
+/**
+ * The check_host() Function as defined in Section 4 of RFC4408
+ * @param self SidfRequest object.
+ * @param domain <domain> parameter of the check_host() function
+ */
 static SidfScore
 SidfRequest_checkHost(SidfRequest *self, const char *domain)
 {
@@ -1108,7 +1148,7 @@ SidfRequest_checkHost(SidfRequest *self, const char *domain)
     SidfRequest_popDomain(self);
     SidfRecord_free(record);
     return eval_score;
-}   // end function : SidfRequest_checkHost
+}   // end function: SidfRequest_checkHost
 
 /**
  * HELO は指定必須. sender が指定されていない場合, postmaster@(HELOとして指定したドメイン) を sender として使用する.
@@ -1144,14 +1184,14 @@ SidfRequest_eval(SidfRequest *self, SidfRecordScope scope)
     self->redirect_depth = 0;
     self->include_depth = 0;
     return SidfRequest_checkHost(self, InetMailbox_getDomain(self->sender));
-}   // end function : SidfRequest_eval
+}   // end function: SidfRequest_eval
 
 /**
  * This function sets an IP address to the SidfRequest object via sockaddr structure.
  * The IP address is used as <ip> parameter of check_host function.
  * @param self SidfRequest object.
  * @param sa_family address family. AF_INET for IPv4, AF_INET6 for IPv6.
- * @param addr pointer to the sockaddr_in structure for IPv4,
+ * @param addr a pointer to the sockaddr_in structure for IPv4,
  *             sockaddr_in6 structure for IPv6.
  * @return true on successful completion, false otherwise.
  *         If sa_family is specified correctly, this function won't fail.
@@ -1175,7 +1215,7 @@ SidfRequest_setIpAddr(SidfRequest *self, sa_family_t sa_family, const struct soc
     default:
         return false;
     }   // end switch
-}   // end function : SidfRequest_setIpAddr
+}   // end function: SidfRequest_setIpAddr
 
 /**
  * This function sets an IP address to the SidfRequest object with string representation.
@@ -1195,13 +1235,13 @@ SidfRequest_setIpAddrString(SidfRequest *self, sa_family_t sa_family, const char
     self->sa_family = sa_family;
     switch (sa_family) {
     case AF_INET:
-        return (1 == inet_pton(AF_INET, address, &(self->ipaddr.addr4))) ? true : false;
+        return bool_cast(1 == inet_pton(AF_INET, address, &(self->ipaddr.addr4)));
     case AF_INET6:
-        return (1 == inet_pton(AF_INET6, address, &(self->ipaddr.addr6))) ? true : false;
+        return bool_cast(1 == inet_pton(AF_INET6, address, &(self->ipaddr.addr6)));
     default:
         return false;
     }   // end switch
-}   // end function : SidfRequest_setIpAddrString
+}   // end function: SidfRequest_setIpAddrString
 
 /**
  * 送信者のメールアドレスを SidfRequest にセットする.
@@ -1227,7 +1267,7 @@ SidfRequest_setSender(SidfRequest *self, const InetMailbox *sender)
 
     self->sender = mailbox;
     return true;
-}   // end function : SidfRequest_setSender
+}   // end function: SidfRequest_setSender
 
 /**
  * HELO ドメインを SidfRequest にセットする.
@@ -1247,7 +1287,7 @@ SidfRequest_setHeloDomain(SidfRequest *self, const char *domain)
     free(self->helo_domain);
     self->helo_domain = tmp;
     return true;
-}   // end function : SidfRequest_setHeloDomain
+}   // end function: SidfRequest_setHeloDomain
 
 void
 SidfRequest_reset(SidfRequest *self)
@@ -1259,7 +1299,6 @@ SidfRequest_reset(SidfRequest *self)
     if (NULL != self->domain) {
         StrArray_reset(self->domain);
     }   // end if
-    self->sender = NULL;
     self->dns_mech_count = 0;
     self->is_sender_context = false;
     self->local_policy_mode = false;
@@ -1278,8 +1317,12 @@ SidfRequest_reset(SidfRequest *self)
         free(self->explanation);
         self->explanation = NULL;
     }   // end if
-}   // end function : SidfRequest_reset
+}   // end function: SidfRequest_reset
 
+/**
+ * release SidfRequest object
+ * @param self SidfRequest object to release
+ */
 void
 SidfRequest_free(SidfRequest *self)
 {
@@ -1300,8 +1343,12 @@ SidfRequest_free(SidfRequest *self)
         free(self->explanation);
     }   // end if
     free(self);
-}   // end function : SidfRequest_free
+}   // end function: SidfRequest_free
 
+/**
+ * create SidfRequest object
+ * @return initialized SidfRequest object, or NULL if memory allocation failed.
+ */
 SidfRequest *
 SidfRequest_new(const SidfPolicy *policy, DnsResolver *resolver)
 {
@@ -1327,4 +1374,4 @@ SidfRequest_new(const SidfPolicy *policy, DnsResolver *resolver)
   cleanup:
     SidfRequest_free(self);
     return NULL;
-}   // end function : SidfRequest_new
+}   // end function: SidfRequest_new
