@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2008-2009 Internet Initiative Japan Inc. All rights reserved.
+ * Copyright (c) 2006-2011 Internet Initiative Japan Inc. All rights reserved.
  *
  * The terms and conditions of the accompanying program
  * shall be provided separately by Internet Initiative Japan Inc.
  * Any use, reproduction or distribution of the program are permitted
  * provided that you agree to be bound to such terms and conditions.
  *
- * $Id: xskip.c 606 2009-02-01 17:25:57Z takahiko $
+ * $Id: xskip.c 1365 2011-10-16 08:08:36Z takahiko $
  */
 
 #include "rcsid.h"
-RCSID("$Id: xskip.c 606 2009-02-01 17:25:57Z takahiko $");
+RCSID("$Id: xskip.c 1365 2011-10-16 08:08:36Z takahiko $");
 
 #include <sys/types.h>
 #include <string.h>
@@ -241,7 +241,7 @@ XSkip_atextBlock(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_ATEXT(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_atextBlock
+}   // end function: XSkip_atextBlock
 
 /*
  * [RFC2822]
@@ -261,7 +261,7 @@ XSkip_atom(const char *head, const char *tail, const char **nextp)
     XSkip_cfws(p, tail, &p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_atom
+}   // end function: XSkip_atom
 
 /*
  * [RFC2822]
@@ -277,10 +277,10 @@ XSkip_dtextBlock(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_DTEXT(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_dtextBlock
+}   // end function: XSkip_dtextBlock
 
 /*
- * [RFC2045]
+ * [RFC2045] 5.1.
  * token := 1*<any (US-ASCII) CHAR except SPACE, CTLs,
  *             or tspecials>
  * tspecials :=  "(" / ")" / "<" / ">" / "@" /
@@ -290,13 +290,31 @@ XSkip_dtextBlock(const char *head, const char *tail, const char **nextp)
  *               ; to use within parameter values
  */
 int
-XSkip_mimeTokenBlock(const char *head, const char *tail, const char **nextp)
+XSkip_mimeToken(const char *head, const char *tail, const char **nextp)
 {
     const char *p;
     for (p = head; p < tail && IS_MIMETOKEN(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_mimeTokenBlock
+}   // end function: XSkip_mimeToken
+
+/*
+ * [RFC2045] 5.1.
+ * value := token / quoted-string
+ */
+int
+XSkip_mimeValue(const char *head, const char *tail, const char **nextp)
+{
+    const char *p;
+    if (0 < XSkip_mimeToken(head, tail, &p)
+        || 0 < XSkip_2822QuotedString(head, tail, &p)) {
+        *nextp = p;
+    } else {
+        *nextp = head;
+    }   // end if
+
+    return *nextp - head;
+}   // end function: XSkip_mimeValue
 
 /*
  * 指定した任意の 1 文字をスキップする関数
@@ -306,7 +324,7 @@ XSkip_char(const char *head, const char *tail, char c, const char **nextp)
 {
     *nextp = (head < tail && *head == c) ? head + 1 : head;
     return *nextp - head;
-}   // end function : XSkip_char
+}   // end function: XSkip_char
 
 /*
  * 指定した任意の文字列をスキップする関数
@@ -323,7 +341,7 @@ XSkip_string(const char *head, const char *tail, const char *str, const char **n
     }   // end if
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_string
+}   // end function: XSkip_string
 
 /*
  * 指定した任意の文字列を case-insensitive にスキップする関数
@@ -340,7 +358,7 @@ XSkip_casestring(const char *head, const char *tail, const char *str, const char
     }   // end if
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_casestring
+}   // end function: XSkip_casestring
 
 /*
  * [RFC2822]
@@ -354,7 +372,7 @@ XSkip_ctext(const char *head, const char *tail, const char **nextp)
 {
     *nextp = (head < tail && IS_CTEXT(*head)) ? head + 1 : head;
     return *nextp - head;
-}   // end function : XSkip_ctext
+}   // end function: XSkip_ctext
 
 /*
  * [RFC2822]
@@ -368,7 +386,7 @@ XSkip_quotedPair(const char *head, const char *tail, const char **nextp)
         *nextp += 2;
     }   // end if
     return *nextp - head;
-}   // end function : XSkip_quotedPair
+}   // end function: XSkip_quotedPair
 
 /*
  * [RFC2822]
@@ -384,7 +402,7 @@ XSkip_qcontent(const char *head, const char *tail, const char **nextp)
 
     // *s が qtext でない場合は XSkip_quotedPair() にそのまま委譲
     return XSkip_quotedPair(head, tail, nextp);
-}   // end function : XSkip_qcontent
+}   // end function: XSkip_qcontent
 
 /*
  * [RFC2822]
@@ -400,7 +418,7 @@ XSkip_dcontent(const char *head, const char *tail, const char **nextp)
 
     // *s が dtext でない場合は XSkip_quotedPair() にそのまま委譲
     return XSkip_quotedPair(head, tail, nextp);
-}   // end function : XSkip_dcontent
+}   // end function: XSkip_dcontent
 
 /*
  * [RFC2822]
@@ -423,7 +441,7 @@ XSkip_dotAtomText(const char *head, const char *tail, const char **nextp)
     }   // end while
 
     return *nextp - head;
-}   // end function : XSkip_dotAtomText
+}   // end function: XSkip_dotAtomText
 
 /*
  * [RFC2822]
@@ -440,7 +458,7 @@ XSkip_looseDotAtomText(const char *head, const char *tail, const char **nextp)
     while (0 < XSkip_atextBlock(p, tail, &p) || 0 < XSkip_char(p, tail, '.', &p));
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_looseDotAtomText
+}   // end function: XSkip_looseDotAtomText
 
 /*
  * [RFC2822]
@@ -460,7 +478,7 @@ XSkip_dotAtom(const char *head, const char *tail, const char **nextp)
     XSkip_cfws(p, tail, &p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_dotAtom
+}   // end function: XSkip_dotAtom
 
 /*
  * [RFC2822]
@@ -482,7 +500,7 @@ XSkip_looseDotAtom(const char *head, const char *tail, const char **nextp)
     XSkip_cfws(p, tail, &p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_looseDotAtom
+}   // end function: XSkip_looseDotAtom
 
 /*
  * [RFC2822]
@@ -508,7 +526,7 @@ XSkip_domainLiteral(const char *head, const char *tail, const char **nextp)
     XSkip_cfws(p, tail, &p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_domainLiteral
+}   // end function: XSkip_domainLiteral
 
 /*
  * [RFC2822]
@@ -529,7 +547,7 @@ XSkip_2822Domain(const char *head, const char *tail, const char **nextp)
     }   // end if
 
     return *nextp - head;
-}   // end function : XSkip_2822Domain
+}   // end function: XSkip_2822Domain
 
 /*
  * [RFC2822]
@@ -557,7 +575,7 @@ XSkip_2822QuotedString(const char *head, const char *tail, const char **nextp)
     XSkip_cfws(p, tail, &p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_2822QuotedString
+}   // end function: XSkip_2822QuotedString
 
 /*
  * [RFC2822]
@@ -578,7 +596,7 @@ XSkip_word(const char *head, const char *tail, const char **nextp)
     }   // end if
 
     return *nextp - head;
-}   // end function : XSkip_word
+}   // end function: XSkip_word
 
 /*
  * [RFC2822]
@@ -590,7 +608,7 @@ XSkip_phrase(const char *head, const char *tail, const char **nextp)
     *nextp = head;
     while (0 < XSkip_word(*nextp, tail, nextp));
     return *nextp - head;
-}   // end function : XSkip_phrase
+}   // end function: XSkip_phrase
 
 /*
  * [RFC2822]
@@ -611,7 +629,7 @@ XSkip_2822LocalPart(const char *head, const char *tail, const char **nextp)
     }   // end if
 
     return *nextp - head;
-}   // end function : XSkip_2822LocalPart
+}   // end function: XSkip_2822LocalPart
 
 /*
  * [RFC2822]
@@ -631,7 +649,7 @@ XSkip_addrSpec(const char *head, const char *tail, const char **nextp)
 
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_addrSpec
+}   // end function: XSkip_addrSpec
 
 /*
  * [RFC2822]
@@ -653,7 +671,7 @@ XSkip_ccontent(const char *head, const char *tail, const char **nextp)
     }   // end if
 
     return *nextp - head;
-}   // end function : XSkip_ccontent
+}   // end function: XSkip_ccontent
 
 /*
  * [RFC2822]
@@ -680,7 +698,7 @@ XSkip_comment(const char *head, const char *tail, const char **nextp)
 
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_comment
+}   // end function: XSkip_comment
 
 /*
  * [RFC2822]
@@ -697,7 +715,7 @@ XSkip_cfws(const char *head, const char *tail, const char **nextp)
 
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_cfws
+}   // end function: XSkip_cfws
 
 /*
  * [RFC5234]
@@ -722,7 +740,7 @@ XSkip_crlf(const char *head, const char *tail, const char **nextp)
     }   // end if
 
     return *nextp - head;
-}   // end function : XSkip_crlf
+}   // end function: XSkip_crlf
 
 /*
  * [RFC5234]
@@ -740,7 +758,7 @@ XSkip_crlfBlock(const char *head, const char *tail, const char **nextp)
     while (0 < XSkip_crlf(p, tail, &p));
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_crlfBlock
+}   // end function: XSkip_crlfBlock
 
 /*
  * [RFC2822]
@@ -756,7 +774,7 @@ XSkip_fieldName(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_FTEXT(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_fieldName
+}   // end function: XSkip_fieldName
 
 /*
  * [RFC5234]
@@ -780,7 +798,7 @@ XSkip_lwsp(const char *head, const char *tail, const char **nextp)
         XSkip_crlf(p, tail, &p);
     } while (0 < XSkip_wsp(p, tail, &p));
     return *nextp - head;
-}   // end function : XSkip_lwsp
+}   // end function: XSkip_lwsp
 
 /*
  * SPBLOCK =  *SP
@@ -792,7 +810,7 @@ XSkip_spBlock(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_SP(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_spBlock
+}   // end function: XSkip_spBlock
 
 /*
  * [RFC5234]
@@ -804,7 +822,7 @@ XSkip_wsp(const char *head, const char *tail, const char **nextp)
 {
     *nextp = (head < tail && IS_WSP(*head)) ? head + 1 : head;
     return *nextp - head;
-}   // end function : XSkip_wsp
+}   // end function: XSkip_wsp
 
 /*
  * [RFC5234]
@@ -818,7 +836,7 @@ XSkip_wspBlock(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_WSP(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_wspBlock
+}   // end function: XSkip_wspBlock
 
 /*
  * [RFC5234]
@@ -832,7 +850,7 @@ XSkip_digitBlock(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_DIGIT(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_digitBlock
+}   // end function: XSkip_digitBlock
 
 /*
  * [RFC5234]
@@ -843,7 +861,7 @@ XSkip_alpha(const char *head, const char *tail, const char **nextp)
 {
     *nextp = (head < tail && IS_ALPHA(*head)) ? head + 1 : head;
     return *nextp - head;
-}   // end function : XSkip_alpha
+}   // end function: XSkip_alpha
 
 /*
  * ALNUMBLOCK =  *(ALPHA / DIGIT)
@@ -855,11 +873,11 @@ XSkip_alnumBlock(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_ALNUM(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_alnumBlock
+}   // end function: XSkip_alnumBlock
 
 /*
- * [RFC4871]
- * tag-name  =  ALPHA 0*ALNUMPUNC
+ * [RFC6376]
+ * tag-name  =  ALPHA *ALNUMPUNC
  * ALNUMPUNC =  ALPHA / DIGIT / "_"
  */
 int
@@ -876,10 +894,10 @@ XSkip_tagName(const char *head, const char *tail, const char **nextp)
     for (++p; p < tail && IS_ALNUMPUNC(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_tagName
+}   // end function: XSkip_tagName
 
 /*
- * [RFC4871]
+ * [RFC6376]
  * tval      =  1*VALCHAR
  * VALCHAR   =  %x21-3A / %x3C-7E
  *                   ; EXCLAMATION to TILDE except SEMICOLON
@@ -891,12 +909,12 @@ XSkip_tval(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_VALCHAR(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_tval
+}   // end function: XSkip_tval
 
 /*
- * [RFC4871]
- * tag-value =  [ tval 0*( 1*(WSP / FWS) tval ) ]
- *                   ; WSP and FWS prohibited at beginning and end
+ * [RFC6376]
+ * tag-value =  [ tval *( 1*(WSP / FWS) tval ) ]
+ *                   ; Prohibits WSP and FWS at beginning and end
  */
 int
 XSkip_tagValue(const char *head, const char *tail, const char **nextp)
@@ -908,7 +926,7 @@ XSkip_tagValue(const char *head, const char *tail, const char **nextp)
         while (0 < XSkip_wsp(p, tail, &p) || 0 < XSkip_fws(p, tail, &p));
     }   // end while
     return *nextp - head;
-}   // end function : XSkip_tagValue
+}   // end function: XSkip_tagValue
 
 /*
  * [RFC4408]
@@ -928,7 +946,7 @@ XSkip_spfName(const char *head, const char *tail, const char **nextp)
     for (++p; p < tail && IS_SPF_NAME(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_spfName
+}   // end function: XSkip_spfName
 
 /*
  * [RFC2821]
@@ -961,7 +979,7 @@ XSkip_2821QuotedString(const char *head, const char *tail, const char **nextp)
 
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_2821QuotedString
+}   // end function: XSkip_2821QuotedString
 
 /*
  * [RFC2821]
@@ -973,7 +991,7 @@ XSkip_dotString(const char *head, const char *tail, const char **nextp)
 {
     // delegate to XSkip_dotAtomText() for equivalence between Dot-string and dot-atom-text
     return XSkip_dotAtomText(head, tail, nextp);
-}   // end function : XSkip_dotString
+}   // end function: XSkip_dotString
 
 /*
  * [RFC2821]
@@ -988,7 +1006,7 @@ XSkip_looseDotString(const char *head, const char *tail, const char **nextp)
 {
     // delegate to XSkip_dotAtomText() for equivalence between Dot-string and dot-atom-text
     return XSkip_looseDotAtomText(head, tail, nextp);
-}   // end function : XSkip_looseDotString
+}   // end function: XSkip_looseDotString
 
 /* [RFC2821]
  * Local-part = Dot-string / Quoted-string
@@ -1009,7 +1027,7 @@ XSkip_2821LocalPart(const char *head, const char *tail, const char **nextp)
     }   // end if
 
     return *nextp - head;
-}   // end function : XSkip_2821LocalPart
+}   // end function: XSkip_2821LocalPart
 
 /*
  * [RFC2821]
@@ -1063,7 +1081,7 @@ XSkip_addressLiteral(const char *head, const char *tail, const char **nextp)
 
     *nextp = p + 1;
     return *nextp - head;
-}   // end function : XSkip_addressLiteral
+}   // end function: XSkip_addressLiteral
 
 /*
  * [RFC2821]
@@ -1094,10 +1112,10 @@ XSkip_subDomain(const char *head, const char *tail, const char **nextp)
 
     ++(*nextp);
     return *nextp - head;
-}   // end function : XSkip_subDomain
+}   // end function: XSkip_subDomain
 
 /*
- * [RFC4871]
+ * [RFC6376]
  * selector =   sub-domain *( "." sub-domain )
  */
 int
@@ -1117,7 +1135,7 @@ XSkip_selector(const char *head, const char *tail, const char **nextp)
     }   // end while
 
     return *nextp - head;
-}   // end function : XSkip_selector
+}   // end function: XSkip_selector
 
 /*
  * [RFC3461]
@@ -1126,9 +1144,9 @@ XSkip_selector(const char *head, const char *tail, const char **nextp)
 int
 XSkip_realDomain(const char *head, const char *tail, const char **nextp)
 {
-    // delegate to XSkip_selector() for equivalence between real-domain and selector of RFC4871
+    // delegate to XSkip_selector() for equivalence between real-domain and selector of RFC6376
     return XSkip_selector(head, tail, nextp);
-}   // end function : XSkip_realDomain
+}   // end function: XSkip_realDomain
 
 /*
  * [RFC2821]
@@ -1153,7 +1171,7 @@ XSkip_2821Domain(const char *head, const char *tail, const char **nextp)
     } else {
         return XSkip_domainName(p, tail, nextp);
     }   // end if
-}   // end function : XSkip_2821Domain
+}   // end function: XSkip_2821Domain
 
 /*
  * [RFC2821]
@@ -1173,12 +1191,13 @@ XSkip_2821Mailbox(const char *head, const char *tail, const char **nextp)
 
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_2821Mailbox
+}   // end function: XSkip_2821Mailbox
 
 /*
- * [RFC4871]
+ * [RFC6376]
  * domain-name     = sub-domain 1*("." sub-domain)
- *              ; from RFC 2821 Domain, but excluding address-literal
+ *                   ; from [RFC5321] Domain,
+ *                   ; excluding address-literal
  */
 int
 XSkip_domainName(const char *head, const char *tail, const char **nextp)
@@ -1200,7 +1219,7 @@ XSkip_domainName(const char *head, const char *tail, const char **nextp)
     }   // end while
 
     return *nextp - head;
-}   // end function : XSkip_domainName
+}   // end function: XSkip_domainName
 
 static int
 XSkip_base64charBlock(const char *head, const char *tail, const char **nextp)
@@ -1209,12 +1228,12 @@ XSkip_base64charBlock(const char *head, const char *tail, const char **nextp)
     for (p = head; p < tail && IS_BASE64CHAR(*p); ++p);
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_base64charBlock
+}   // end function: XSkip_base64charBlock
 
 /*
- * [RFC4871]
- * base64string =     1*(ALPHA / DIGIT / "+" / "/" / [FWS])
- *                    [ "=" [FWS] [ "=" [FWS] ] ]
+ * [RFC6376]
+ * base64string    =  ALPHADIGITPS *([FWS] ALPHADIGITPS)
+ *                    [ [FWS] "=" [ [FWS] "=" ] ]
  */
 int
 XSkip_base64string(const char *head, const char *tail, const char **nextp)
@@ -1231,7 +1250,7 @@ XSkip_base64string(const char *head, const char *tail, const char **nextp)
     *nextp = p;
 
     return *nextp - head;
-}   // end function : XSkip_base64string
+}   // end function: XSkip_base64string
 
 /*
  * [RFC2822]
@@ -1253,10 +1272,10 @@ XSkip_fws(const char *head, const char *tail, const char **nextp)
         *nextp = p;
     }   // end if
     return *nextp - head;
-}   // end function : XSkip_fws
+}   // end function: XSkip_fws
 
 /*
- * [RFC4871]
+ * [RFC6376]
  * hyphenated-word =  ALPHA [ *(ALPHA / DIGIT / "-") (ALPHA / DIGIT) ]
  */
 int
@@ -1283,12 +1302,14 @@ XSkip_hyphenatedWord(const char *head, const char *tail, const char **nextp)
 
     ++(*nextp);
     return *nextp - head;
-}   // end function : XSkip_hyphenatedWord
+}   // end function: XSkip_hyphenatedWord
 
 /*
- * [RFC4871]
- * x-sig-a-tag-k   = ALPHA *(ALPHA / DIGIT)   ; for later extension
- * x-sig-a-tag-h   = ALPHA *(ALPHA / DIGIT)   ; for later extension
+ * [RFC6376]
+ * x-sig-a-tag-k   = ALPHA *(ALPHA / DIGIT)
+ *                      ; for later extension
+ * x-sig-a-tag-h   = ALPHA *(ALPHA / DIGIT)
+ *                      ; for later extension
  */
 int
 XSkip_alphaAlnum(const char *head, const char *tail, const char **nextp)
@@ -1305,4 +1326,4 @@ XSkip_alphaAlnum(const char *head, const char *tail, const char **nextp)
 
     *nextp = p;
     return *nextp - head;
-}   // end function : XSkip_alphaAlnum
+}   // end function: XSkip_alphaAlnum
